%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%% Copyright (C) 1992 Ellemtel Telecommunications Systems Laboratories
%% File    : dbg_imsg.erl
%% Author  : Magnus Fr|berg
%% Purpose : Erlang meta evaluator

-module(dbg_imsg).
-copyright('Copyright (c) 1991-97 Ericsson Telecom AB').
-vsn('$Revision: /main/release/free/1').

%% -- Exported user functions.

-export([start/1,msg/1]).

start(Exprs) ->
    Meta = spawn_link(dbg_imeta,int,[self(),Exprs]),
    msg(Meta).

msg(Meta) ->
    case catch msg1(Meta) of
	{ready,Meta,Value} ->
	    Value;
	{'EXIT',Reason} ->
	    Meta ! {sys,self(),{exited_nocatch,Reason}},
	    wait_exit(Meta);
	Throwed ->
	    Meta ! {sys,self(),{throwed_nocatch,Throwed}},
	    throw(Throwed)
    end.


%%---------------------------------------------------
%%-- Sync on exit.
%%-- The Meta process shall initiate all exits !!
%%---------------------------------------------------

wait_exit(Meta) ->
    receive
	{sys,Meta,{exit,Reason}} ->
	    exit(Reason);
	{'EXIT',Meta,Reason} ->
	    exit(Reason)
    end.

msg1(Meta) ->
    case wait_msg(Meta) of
	{ready,Value} ->
	    {ready,Meta,Value};
	_ ->
	    msg1(Meta)
    end.

wait_msg(Meta) ->
    receive
	{sys,Meta,Command} ->
	    handle_command(Meta,Command);
	{'EXIT',Meta,Reason} ->
	    exit(Reason)
    end.

handle_command(Meta, {ready,Val}) ->
    {ready,Val};
handle_command(Meta, {'receive',Msg}) ->
    receive
	Msg -> 
	    Meta ! {self(),rec_acked},
	    true
    end;
handle_command(Meta, {exit,Reason}) ->
    exit(Reason);
handle_command(Meta, {bif,Name,As}) ->
    Res = dbg_ieval:bif(Name,As),
    Meta ! {sys,self(),{bif,Res}},
    true;
handle_command(Meta, {bif,Name,As,Followed}) ->
    Res = dbg_ieval:bif(Name,As,Followed),
    Meta ! {sys,self(),{bif,Res}},
    true;
handle_command(Meta, {catch_bif,Name,As}) ->
    case catch catch_bif(Meta,Name,As) of
	{catch_normal,Meta,Res} ->
	    Meta ! {sys,self(),{bif,Res}},
	    true;
	Throwed ->   % Does also cover exits !!
	    Meta ! {sys,self(),{throwed,Throwed}},
	    true
    end;
handle_command(Meta, {catch_bif,Name,As,Followed}) ->
    case catch catch_bif(Meta,Name,As,Followed) of
	{catch_normal,Meta,Res} ->
	    Meta ! {sys,self(),{bif,Res}},
	    true;
	Throwed ->   % Does also cover exits !!
	    Meta ! {sys,self(),{throwed,Throwed}},
	    true
    end;
handle_command(Meta, {apply,Mod,Fnk,As}) ->
    Res = apply(Mod,Fnk,As),
    Meta ! {sys,self(),{apply,Res}},
    true;
handle_command(Meta, {catch_apply,Mod,Fnk,As}) ->
    case catch catch_apply(Meta,Mod,Fnk,As) of
	{catch_normal,Meta,Res} ->
	    Meta ! {sys,self(),{apply,Res}},
	    true;
	Throwed ->   % Does also cover exits !!
	    Meta ! {sys,self(),{throwed,Throwed}},
	    true
    end;
handle_command(Meta,Command) ->
    io:format('dbg_imsg:handle_command/1 - TBD ! (~w)~n', [Command]),
    true.

%%-- Return tuple if apply evaluates normal, otherwise the
%%-- surrounding catch notices the unnormal exit.

catch_apply(Meta,Mod,Fnk,As) ->
    Res = apply(Mod,Fnk,As),
    {catch_normal,Meta,Res}.

catch_bif(Meta,Name,As) ->
    Res = dbg_ieval:bif(Name,As),
    {catch_normal,Meta,Res}.

catch_bif(Meta,Name,As,Followed) ->
    Res = dbg_ieval:bif(Name,As,Followed),
    {catch_normal,Meta,Res}.
	




