%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%----------------------------------------------------------------------
%%% File    : pman_buf_printer.erl
%%% Author  : Peter Olin <olin@erlang.ericsson.se>
%%% Purpose : 
%%% Created :  3 Jun 1997 by Peter Olin <olin@erlang.ericsson.se>
%%%----------------------------------------------------------------------

-module(pman_buf_printer).
-author('olin@erlang.ericsson.se').

%%-compile(export_all).
-export([init/2]).


-include("pman_buf.hrl").

%% ---------------------------------------------------------------
%% Starts the part of the buffer which regulates the flow of data to
%% be printed in the pid editors


init(Editor,Buffer_pid) ->
    Buffer_pid!{printer,send},
    printer_loop(Editor,Buffer_pid).
    
printer_loop(Editor,Buffer_pid)->
    receive
	{save_buffer,Name} ->
	    gs:config(Editor,{save,Name}),
	    TT = "Buffer saved in file\n" ++ Name,
	    pman_win:dialog_window(gs:start(),TT),
	    printer_loop(Editor,Buffer_pid);
	{buffer,Trace} ->
	    case lists:flat_length(Trace) of
		Len when Len > ?MAX_OUTPUT ->
		    printer_long(lists:flatten(Trace),Editor),
		    Buffer_pid!{printer,send},
		    printer_loop(Editor,Buffer_pid);
		_ ->
		    		    Buffer_pid!{printer,send},
		    print_trace(Editor,Trace),
		    printer_loop(Editor,Buffer_pid)
	    end;
	clear ->
	    pman_win:configeditor(Editor, [{enable, true}]),
	    pman_win:configeditor(Editor,clear),
	    pman_win:configeditor(Editor, [{enable, false}]),
	    printer_loop(Editor,Buffer_pid);
	Other ->
	    printer_loop(Editor,Buffer_pid)
    end.

printer_long([],_) -> ok;
printer_long(Trace,Editor) ->
    receive
	clear ->
	    pman_win:configeditor(Editor, [{enable, true}]),
	    pman_win:configeditor(Editor,clear),
	    pman_win:configeditor(Editor, [{enable, false}])
    after 0 ->
	    {Length,Rest,Print} = pman_buf_utils:split(Trace,
						       ?MAX_OUTPUT,
						       0,
						       []),
	    print_trace(Editor,Print),
	    printer_long(Rest,Editor)
    end.



%% ---------------------------------------------------------------
%% Function which print trace messages on the window 
%% ---------------------------------------------------------------

print_trace(Editor,Elements) ->
    pman_win:configeditor(Editor, [{enable, true}]),
    pman_win:configeditor(Editor, [{insert, {'end',Elements}}]),
    pman_win:configeditor(Editor, [{enable, false}]).	
