%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%----------------------------------------------------------------------
%%% File    : pman_relay_server.erl
%%% Author  : Peter Olin <olin@erlang.ericsson.se>
%%% Purpose : Relay server code.
%%% Created : 31 Oct 1997 by Peter Olin <olin@erlang.ericsson.se>
%%%----------------------------------------------------------------------

-module(pman_relay_server).
-author('olin@erlang.ericsson.se').

%%-compile(export_all).
-export([init/1]).



init(P) ->
    process_flag(trap_exit, true),

    loop(P).

loop(P) ->
    receive
	{ok_to_trace, PidSender, PidToTrace} ->

	    case  catch erlang:trace(PidToTrace, false, [send]) of
		true ->

		    PidSender ! {ok_to_trace, self()},
		    loop(P);
		
		_Otherwise ->

		    PidSender ! {not_ok_to_trace, self()}
	    end;

	{P, M,F, A} ->

	    case catch apply(M,F,A) of
		true -> ok;
		_ ->  P ! {print, "** Illegal trace request **\n", []}
	    end,
	    loop(P);
	{'EXIT', P, Reason} ->

	    exit(normal);
	Other ->             %% Here is the normal case for trace i/o

	    P ! Other, 
	    loop(P)
    end.




