%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%*********************************************************************
%%%
%%%   Filename:         tv_db.erl
%%%
%%%   Author:           EPK/TE  Fredrik Gustafson 
%%% 
%%%   SCCS Delta ID:    1.3
%%%
%%%   SCCS Delta Date:  97/06/12
%%%
%%%   SCCS ID:          @(#)  tv_db.erl  1.3  97/06/12
%%%
%%%   Description:      Module handling the internal database in the table tool.
%%%
%%%*********************************************************************
%%%
%%%   COPYRIGHT ERICSSON TELECOM AB 1998
%%%
%%%   THE COPYRIGHT TO THE COMPUTER PROGRAM HEREIN IS THE
%%%   PROPERTY OF ERICSSON TELECOM AB, SWEDEN. THE PROGRAM
%%%   MAY BE USED AND/OR COPIED ONLY WITH THE WRITTEN PERMISSION
%%%   FROM ERICSSON TELECOM AB, OR IN ACCORDANCE WITH THE TERMS
%%%   AND CONDITIONS STIPULATED IN THE AGREEMENT/CONTRACT UNDER
%%%   WHICH THE PROGRAM HAS BEEN SUPPLIED.
%%%
%%%*********************************************************************
%%%
%%%   Revision information:
%%%
%%%   Date      Name                       Description
%%%   ------    -----------------------    -----------
%%%   970427    EPK/TE  F Gustafson        First version.
%%%
%%%*********************************************************************



-module(tv_db).
-author('fredrik@erlang.ericsson.se').
-vsn('$Revision: /main/release/free/1').
-copyright('Copyright (C) 1991-98 Ericsson Telecom AB').



-export([dbs/1]).



-include("tv_int_def.hrl").
-include("tv_int_msg.hrl").
-include("tv_db_int_def.hrl").







%%%*********************************************************************
%%% EXTERNAL FUNCTIONS
%%%*********************************************************************




dbs(Master) ->
    process_flag(trap_exit, true),
    ProcVars = #process_variables{master_pid = Master},
    blocked(ProcVars).









%%%********************************************************************
%%% INTERNAL FUNCTIONS
%%%********************************************************************




%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


blocked(ProcVars) ->
    receive
        Msg ->
            case Msg of 

                #dbs_deblock{} ->
                    deblock(Msg, ProcVars, false);

                {'EXIT', Pid, Reason} ->
                    MasterPid = ProcVars#process_variables.master_pid,
                    exit_signals({Pid, Reason}, MasterPid),
                    blocked(ProcVars);

                _Other ->
                    blocked(ProcVars)
            end
    end.








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


deblock(Msg, ProcVars, SearchWinCreated) ->
    #dbs_deblock{sender         = Sender,
		 type           = Type,
		 keypos         = KeyPos,
		 sublist_length = SublistLength} = Msg,

    NewDbData = #db_data{subset_size = SublistLength,
			 subset_pos  = 1,
			 key_no      = KeyPos,
			 ets_type    = Type
			},
    NewProcVars = ProcVars#process_variables{db_data = NewDbData},
    Sender ! #dbs_deblock_cfm{sender = self()},
    deblocked_loop(NewProcVars, SearchWinCreated, []).
    









%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


deblocked_loop(ProcVars, SearchWinCreated, SearchData) ->
    receive
	Msg ->
	    case Msg of
		
		{gs,entry,keypress,_Data,['Return' | _T]} ->
		    NewSearchData = search_object(ProcVars),
		    deblocked_loop(ProcVars, SearchWinCreated, NewSearchData);

		{gs,entry,keypress,_Data,['Tab' | _T]} ->
		    gs:config(entry, [{select, {0,1000}}]),
		    deblocked_loop(ProcVars, SearchWinCreated, SearchData);
		
		{gs,entry,keypress,_Data,_Args} ->
		    deblocked_loop(ProcVars, SearchWinCreated, SearchData);

		{gs,search,click,_Data,_Args} ->
		    NewSearchData = search_object(ProcVars),
		    deblocked_loop(ProcVars, SearchWinCreated, NewSearchData);
		
		{gs,cancel,click,cancel,_Args} ->
		    tv_db_search:destroy_window(SearchWinCreated),
		    deblocked_loop(ProcVars, false, []);
		
		{gs,listbox,click,_LbData,[Idx | _T]} when SearchData /= [] ->
		    tv_db_search:mark_busy(SearchWinCreated),
		    {Row,_Obj} = lists:nth(Idx+1, SearchData),
		    DbData     = ProcVars#process_variables.db_data,
		       %% Never allow 'subset_pos' to have zero as value! 
		       %% No list can begin with the 0:th element!!! 
		       %% Has to be at least 1!
		    NewDbData  = DbData#db_data{subset_pos=?COMM_FUNC_FILE:max(1,
									       Row),
						subset_size=?ITEMS_TO_DISPLAY},
		    NewProcVars = ProcVars#process_variables{db_data=NewDbData},
		    send_subset(NewProcVars, undefined, undefined),
		    tv_db_search:mark_nonbusy(SearchWinCreated),
		    deblocked_loop(NewProcVars, SearchWinCreated, SearchData);

		{gs,win,configure,_Data,_Args} ->
		    tv_db_search:resize_window(SearchWinCreated),
		    deblocked_loop(ProcVars, SearchWinCreated, SearchData);

		{gs,win,destroy,_Data,_Args} ->
		    deblocked_loop(ProcVars, false, []);


		#dbs_new_data{data = NewData, keys = ListOfKeys, 
			      time_to_read_table = ElapsedTimeEtsread} ->
		    tv_db_search:reset_window(SearchWinCreated),
		    T1 = time(),
		    NewProcVars = update_db(NewData, ListOfKeys, ProcVars),
		    T2 = time(),
		    ElapsedTimeDbs = compute_elapsed_seconds(T1, T2),
		    send_subset(NewProcVars, ElapsedTimeEtsread, ElapsedTimeDbs),
		    deblocked_loop(NewProcVars, SearchWinCreated, []);

		#dbs_subset_req{subset_pos = Pos,subset_length = Length} ->
		    DbData    = ProcVars#process_variables.db_data,
		       %% Never allow 'subset_pos' to have zero as value! 
		       %% No list can begin with the 0:th element!!! 
		       %% Has to be at least 1!
		    NewDbData = DbData#db_data{subset_pos=?COMM_FUNC_FILE:max(1, 
									      Pos),
					       subset_size=Length},
		    NewProcVars = ProcVars#process_variables{db_data = NewDbData},
		    send_subset(NewProcVars, undefined, undefined),
		    deblocked_loop(NewProcVars, SearchWinCreated, SearchData);

		#dbs_marked_row{row_no = RowNo} ->
		    DbData      = ProcVars#process_variables.db_data,
		    NewDbData   = DbData#db_data{requested_row = RowNo},
		    NewProcVars = ProcVars#process_variables{db_data = NewDbData},
		    deblocked_loop(NewProcVars, SearchWinCreated, SearchData);
		    
		#dbs_search_req{} ->
		    tv_db_search:create_window(SearchWinCreated),
		    deblocked_loop(ProcVars, true, SearchData);

		#dbs_sorting_mode{} ->
		    {NewProcVars, NewSearchData} = 
			update_sorting_mode(Msg, ProcVars, 
					    SearchWinCreated, SearchData),
		    deblocked_loop(NewProcVars, SearchWinCreated, NewSearchData);

		#dbs_deblock{} ->
		    tv_db_search:reset_window(SearchWinCreated),
		    deblock(Msg, ProcVars, SearchWinCreated);

                {'EXIT', Pid, Reason} ->
                    MasterPid = ProcVars#process_variables.master_pid,
                    exit_signals({Pid, Reason}, MasterPid),
                    deblocked_loop(ProcVars, SearchWinCreated, SearchData);

                _Other ->
                       %% io:format("Received message: ~w ~n", [_Other]),
                    deblocked_loop(ProcVars, SearchWinCreated, SearchData)
	    end
    end.






search_object(ProcVars) ->
    DbData = ProcVars#process_variables.db_data,
    DbList = dblist2list(DbData#db_data.db),
    case catch tv_db_search:get_pattern_and_search(DbList) of
	{'EXIT', Reason} ->
	    tv_db_search:reset_window(true),
	    [];
	List ->
	    List
    end.





%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


update_sorting_mode(Msg, ProcVars, SearchWinCreated, OldSearchData) ->
    #dbs_sorting_mode{sorting     = Sorting,
		      reverse     = Reverse,
		      sort_key_no = SortKeyNo} = Msg,

    DbData = ProcVars#process_variables.db_data,

    #db_data{db          = DbList,
	     sorting     = OldSorting,
	     rev_sorting = OldReverse,
	     sort_key_no = OldSortKeyNo} = DbData,
    

    NewDbList = sort_db_list(DbList, Sorting, OldSorting, Reverse, OldReverse, 
			     SortKeyNo, OldSortKeyNo),
    
    NewDbData = DbData#db_data{db          = NewDbList,
			       sorting     = Sorting,
			       rev_sorting = Reverse,
			       sort_key_no = SortKeyNo
			      },

    NewProcVars = ProcVars#process_variables{db_data = NewDbData},
    send_subset(NewProcVars, undefined, undefined),

    SearchData = 
	case Sorting of
	    false ->
		OldSearchData;
	    OldSorting when Reverse == OldReverse,
			    SortKeyNo == OldSortKeyNo ->
		[];
	    OldSorting when Reverse == OldReverse,
			    OldSortKeyNo == undefined->
		[];
	    _Other ->
		case catch tv_db_search:update_search(SearchWinCreated, 
						      NewDbList) of
		    {'EXIT', Reason} ->
			tv_db_search:reset_window(true),
			[];
		    List ->
			List
		end
	end,

    {NewProcVars, SearchData}.






%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


sort_db_list(DbList, Sort, Sort, Rev, Rev, KeyNo, KeyNo) ->
       % Already sorted!
    DbList;
sort_db_list(DbList, false, OldSort, Rev, OldRev, KeyNo, OldKeyNo) ->
       % No sorting, i.e., the old list order suffices!
    DbList;
sort_db_list(DbList, Sort, OldSort, Rev, OldRev, KeyNo, OldKeyNo) ->
    tv_db_sort:mergesort(KeyNo, DbList, Rev).
    











%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


send_subset(ProcVars, EtsreadTime, DbsTime) ->
    #process_variables{master_pid    = MasterPid,
		       db_data       = DbData,
		       list_of_keys  = ListOfKeys}   = ProcVars,
    
    #db_data{subset_size   = SubsetSize,
	     subset_pos    = SubsetPos,
	     requested_row = RowNo,
	     db_size       = DbSize,
	     db            = DbList,
	     max_elem_size = MaxElemSize}  = DbData,
    

    RowData = get_requested_row_data(RowNo, DbList),

    if 
	DbSize > 0 ->
	    Pos = ?COMM_FUNC_FILE:min(SubsetPos, DbSize),
	       % Requested_data may be shorter than requested, but that's OK,
               % pd handles that correctly!
	    Subset = lists:sublist(DbList, Pos, SubsetSize),
	    MasterPid ! #dbs_subset{sender                = self(),
				    data                  = Subset,
				    subset_pos            = Pos,
				    db_length             = DbSize,
				    list_of_keys          = ListOfKeys,
				    max_elem_size         = MaxElemSize,
				    requested_row         = RowData,
				    required_time_etsread = EtsreadTime,
				    required_time_dbs     = DbsTime
				   };
	true ->
	    MasterPid ! #dbs_subset{sender                = self(),
				    data                  = [],
				    subset_pos            = 1,
				    db_length             = 0,
				    list_of_keys          = ListOfKeys,
				    max_elem_size         = MaxElemSize,
				    requested_row         = RowData,
				    required_time_etsread = EtsreadTime,
				    required_time_dbs     = DbsTime
				   }
    end.
    
    








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


get_requested_row_data(undefined, DbList) ->
    [];
get_requested_row_data(RowNo, []) ->
    [];
get_requested_row_data(RowNo, DbList) ->
    case catch lists:nth(RowNo, DbList) of
	{'EXIT', Reason} ->
	    [];
	RowData ->
	    [RowData]
    end.











%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


exit_signals(ExitInfo, MasterPid) ->
    case ExitInfo of
        {MasterPid, _Reason} ->
               % When from master, just quit!
            exit(normal);
        _Other ->
            done
    end.













%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


update_db(NewList, ListOfKeys, ProcVars) ->
    DbData = ProcVars#process_variables.db_data,
    #db_data{db            = OldDbList,
	     db_size       = DbSize,
	     max_elem_size = MaxElemSize,
	     deleted       = DelList,
	     ets_type      = EtsType,
	     sorting       = Sorting,
	     rev_sorting   = RevSorting,
	     sort_key_no   = SortKeyNo,
	     key_no        = KeyNo} = DbData,
    
    DbList   = update_colors(OldDbList -- DelList),
    OldList  = dblist2list(DbList),
    InsOrUpd = (NewList -- OldList),
    DelOrUpd = (OldList -- NewList),
    
    {Inserted, Deleted, Updated} = group_difflists(EtsType, KeyNo, InsOrUpd, 
						   DelOrUpd),
    DelMarked = mark_deleted(KeyNo, Deleted, DbList),
    Replaced  = replace_elements(KeyNo, Updated, DelMarked),
    NewDbList = add_elements(KeyNo, Inserted, Replaced, Sorting, RevSorting, 
			     SortKeyNo),

    NewMaxSize = ?COMM_FUNC_FILE:max(MaxElemSize, 
				     ?COMM_FUNC_FILE:max(max_size(Replaced), 
							 max_size(Inserted))),

    NewDbData = DbData#db_data{db            = NewDbList,
			       db_size       = length(NewDbList),
			       max_elem_size = NewMaxSize,
			       deleted       = list2dblist(Deleted, ?BLACK)
			      },

%     io:format("~n~nNew elements: ~p ~n", [Inserted]),
%     io:format("Updated elements: ~p ~n", [Updated]),
%     io:format("Deleted elements: ~p ~n~n", [Deleted]),
%     io:format("Total list: ~p ~n", [NewDbList]),

    ProcVars#process_variables{db_data = NewDbData,
			       list_of_keys = ListOfKeys
			      }.
    











%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


max_size([]) ->    
    0;
max_size(L) ->
    max_size(L, 0).








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


max_size([], CurrMax) ->
    CurrMax;
max_size([H | T], CurrMax) ->
    Size = if 
	       tuple(H) ->
		   size(H);
	       true ->
		   1
	   end,
    if
	Size >= CurrMax ->
	    max_size(T, Size);
	true ->
	    max_size(T, CurrMax)
    end.













%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


add_elements(KeyNo, Inserted, List, false, _RevSorting, _SortKeyNo) ->
       % Remember that the order of the original list has to be preserved!
    List ++ list2dblist(Inserted, ?RED1);
add_elements(_KeyNo, Inserted, List, Sorting, RevSorting, SortKeyNo) ->
       % The original list i already sorted - sort the new elements, and
       % just merge the two lists together!
    SortedInsertedList = tv_db_sort:mergesort(SortKeyNo, 
					       list2dblist(Inserted, ?RED1), 
					       RevSorting),
    tv_db_sort:merge(SortKeyNo, List, SortedInsertedList, RevSorting).
    












%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


%% We assume the list already has been sorted, i.e., since the order won't 
%% be changed by marking an element deleted, we DON'T have to sort the list
%% once again!

mark_deleted(_KeyNo, [], List) ->
    List;
mark_deleted(KeyNo, [Data | T], List) ->
    KeyValue = tv_db_sort:get_compare_value(KeyNo, Data),
    NewList  = mark_one_element_deleted(KeyNo, KeyValue, Data, List, []),
    mark_deleted(KeyNo, T, NewList).
  
  










%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


mark_one_element_deleted(_KeyNo, _KeyValue, Data, [], Acc) ->
    Acc;
mark_one_element_deleted(KeyNo, {tuple, KeyValue}, 
			 Data, [{DataTuple, Color} | Tail], Acc) ->
    OldKeyValue = tv_db_sort:get_compare_value(KeyNo, DataTuple),
       % Remember that the order of the original list has to be preserved!
    if
	OldKeyValue == {tuple, KeyValue} ->
	    Acc ++ [{Data, ?BLACK}] ++ Tail;
	true ->
	    mark_one_element_deleted(KeyNo, {tuple, KeyValue}, Data, Tail, 
				     Acc ++ [{DataTuple, Color}])
    end;
mark_one_element_deleted(KeyNo, _KeyValue, Data, [{DataTuple, Color} | Tail], Acc) ->
    if
	Data == DataTuple ->
	    Acc ++ [{Data, ?BLACK}] ++ Tail;
	true ->
	    mark_one_element_deleted(KeyNo, _KeyValue, Data, Tail, 
				     Acc ++ [{DataTuple, Color}])
    end.
  
  
  
  





%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


%% We assume the list already has been sorted, i.e., since the order won't 
%% be changed by marking an element updated, we DON'T have to sort the list
%% once again!

replace_elements(_KeyNo, [], List) ->
    List;
replace_elements(KeyNo, [Data | T], List) ->
    KeyValue = tv_db_sort:get_compare_value(KeyNo, Data),
    NewList  = replace_one_element(KeyNo, KeyValue, Data, List, []),
    replace_elements(KeyNo, T, NewList).





    
    





%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


replace_one_element(_KeyNo, _Key, _Data, [], Acc) ->
    Acc;
replace_one_element(KeyNo, {tuple, Key1}, Data, [{DataTuple, Color} | Tail], Acc) ->
    Key2 = tv_db_sort:get_compare_value(KeyNo, DataTuple),
    % Remember that the order of the original list has to be preserved!
    if
	Key2 == {tuple, Key1} ->
	    Acc ++ [{Data, ?GREEN1}] ++ Tail;
	true ->
	    replace_one_element(KeyNo, {tuple, Key1}, Data, Tail, 
				Acc ++ [{DataTuple, Color}])
    end;
replace_one_element(KeyNo, _KeyValue, Data, [{DataTuple, Color} | Tail], Acc) ->
       % Can't replace an element with no key!
    Acc ++ [{DataTuple, Color} | Tail].
    













%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


group_difflists(bag, _KeyNo, Inserted, Deleted) ->	     
    % Since the ETS table is of bag type, no element can be updated, i.e.,
    % it can only be deleted and re-inserted, otherwise a new element will be added.
    {Inserted, Deleted, []};
group_difflists(duplicate_bag, _KeyNo, Inserted, Deleted) ->	     
    % Since the ETS table is of duplicate_bag type, no element can be updated, i.e.,
    % it can only be deleted and re-inserted, otherwise a new element will be added.
    {Inserted, Deleted, []};
group_difflists(set, KeyNo, [], Deleted) ->
    % Updated elements has to be present in both lists, i.e., if one list is empty,
    % the other contains no updated elements - they are either inserted or deleted!
    {[], Deleted, []};
group_difflists(set, KeyNo, Inserted, []) ->
    {Inserted, [], []};
group_difflists(set, KeyNo, InsOrUpd, DelOrUpd) ->
    match_difflists(KeyNo, InsOrUpd, DelOrUpd, [], []).
    
    











%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


match_difflists(_KeyNo, [], Deleted, Inserted, Updated) ->
    {Inserted, Deleted, Updated};
match_difflists(KeyNo, [Data | T], DelOrUpd, InsAcc, UpdAcc) ->
    % This function is only called in case of a 'set' ETS table.
    % 'Set' type of ETS table means there are unique keys. If two elements in 
    % InsOrUpd and DelOrUpd have the same key, that element has been updated, 
    % and is added to the Updated list, and removed from the original two lists.
    % After the two lists have been traversed in this way, the remaining elements
    % in DelOrUpd forms the new Deleted list (analogous for InsOrUpd).
    % If we want to improve the performance, we could check which list is the 
    % shortest, since the traversing time depends on this. 
    Key = element(KeyNo, Data),
    case searchdelete(Key, KeyNo, DelOrUpd) of
	{true, NewDelOrUpd} ->
	    match_difflists(KeyNo, T, NewDelOrUpd, InsAcc, [Data | UpdAcc]);
	{false, SameDelOrUpd} ->
	    match_difflists(KeyNo, T, SameDelOrUpd, [Data | InsAcc], UpdAcc)
    end.












%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


searchdelete(_Key, _ElemNo, []) ->
    {false, []};
searchdelete(Key, ElemNo, List) ->
    searchdelete(Key, ElemNo, List, []).











%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


searchdelete(_Key, _ElemNo, [], Acc) ->
    {false, Acc};
searchdelete(Key, ElemNo, [Tuple | Tail], Acc) ->
    % We don't use standard libraries, 'cause we want to make an 'atomic'
    % operation, i.e., we will not search the list two times...
    case (element(ElemNo, Tuple) == Key) of
	true ->
	    {true, Acc ++ Tail};   % Return the list without the matching element
	_Other ->
	    searchdelete(Key, ElemNo, Tail, [Tuple | Acc])
    end.
	
    







%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


dblist2list([]) ->
    [];
dblist2list([{Data, Color} | T]) ->
    [Data | dblist2list(T)].
    
    
    







%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


list2dblist([], Color) ->    
    [];
list2dblist([Data | T], Color) ->
    [{Data, Color} | list2dblist(T, Color)].








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


update_colors([]) ->
    [];
update_colors([{Data, Color} | T]) ->
    [{Data, new_color(Color)} | update_colors(T)].
    









%%======================================================================
%% Function:      new_color.
%%
%% Return Value:  Tuple (triplet) containing RGB-value.
%%
%% Description:   Given a color, a new color is returned. This way one may 
%%                create sequences of color changes.
%%
%% Parameters:    Tuplet (triplet) containing RGB-value.
%%======================================================================


new_color(?GREEN1) ->
    ?GREEN2;
new_color(?GREEN2) ->
    ?GREEN3;
new_color(?GREEN3) ->
    ?GREEN4;
new_color(?GREEN4) ->
    ?GREEN5;
new_color(?GREEN5) ->
    ?DEFAULT_BTN_COLOR;
new_color(?RED1) ->
    ?RED2;
new_color(?RED2) ->
    ?RED3;
new_color(?RED3) ->
    ?RED4;
new_color(?RED4) ->
    ?RED5;
new_color(?RED5) ->
    ?DEFAULT_BTN_COLOR;
new_color(_Other) ->
    ?DEFAULT_BTN_COLOR.                  % Default shall be gray.







%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


compute_elapsed_seconds({H1, M1, S1}, {H2, M2, S2}) ->
    ElapsedHours   = get_time_diff(hours, H1, H2),
    ElapsedMinutes = get_time_diff(minutes, M1, M2),
    ElapsedSeconds = get_time_diff(seconds, S1, S2),
    (ElapsedHours * 3600) + (ElapsedHours * 60) + ElapsedSeconds + 1.








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


get_time_diff(_Type, T1, T2) when T1 =< T2 ->
    T2 - T1;
get_time_diff(hours, T1, T2) ->
    T2 + 24 - T1;
get_time_diff(minutes, T1, T2) ->
    T2 + 60 - T1;
get_time_diff(seconds, T1, T2) ->
    T2 + 60 - T1.
