%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%*********************************************************************
%%%
%%%   Filename:         %M%
%%%
%%%   Author:           EPK/TE  Fredrik Gustafson 
%%% 
%%%   SCCS Delta ID:    %I%
%%%
%%%   SCCS Delta Date:  %E%
%%%
%%%   SCCS ID:          @(#)  %M%  %I%  %E%
%%%
%%%   Description:      
%%%
%%%*********************************************************************
%%%
%%%   COPYRIGHT ERICSSON TELECOM AB 1998
%%%
%%%   THE COPYRIGHT TO THE COMPUTER PROGRAM HEREIN IS THE
%%%   PROPERTY OF ERICSSON TELECOM AB, SWEDEN. THE PROGRAM
%%%   MAY BE USED AND/OR COPIED ONLY WITH THE WRITTEN PERMISSION
%%%   FROM ERICSSON TELECOM AB, OR IN ACCORDANCE WITH THE TERMS
%%%   AND CONDITIONS STIPULATED IN THE AGREEMENT/CONTRACT UNDER
%%%   WHICH THE PROGRAM HAS BEEN SUPPLIED.
%%%
%%%*********************************************************************
%%%
%%%   Revision information:
%%%
%%%   Date      Name                       Description
%%%   ------    -----------------------    -----------
%%%
%%%
%%%*********************************************************************



-module(tv_db_search).
-author('fredrik@erlang.ericsson.se').
-vsn('$Revision: /main/release/free/1').
-copyright('Copyright (C) 1991-98 Ericsson Telecom AB').



-export([create_window/1,
	 resize_window/1,
	 reset_window/1,
	 destroy_window/1,
	 mark_busy/1,
	 mark_nonbusy/1,
	 get_pattern_and_search/1,
	 update_search/2
	]).




-include("tv_int_def.hrl").
-include("tv_int_msg.hrl").
-include("tv_db_int_def.hrl").




-define(WIN_WIDTH, 441).
-define(SMALL_WIN_HEIGHT, 87).
-define(BIG_WIN_HEIGHT, 300).
-define(FRAME_WIDTH, 334).
-define(FRAME_HEIGHT, 70).
-define(FRAME_XPOS, 10-2).
-define(FRAME_YPOS, 10).
-define(ENTRY_XPOS, 9).
-define(ENTRY_YPOS, 26).
-define(ENTRY_WIDTH, ?FRAME_WIDTH-2*?ENTRY_XPOS-5).
-define(LISTBOX_WIDTH, ?WIN_WIDTH-?FRAME_XPOS-(?WIN_WIDTH-(?BTN_XPOS+?BTN_WIDTH))+6).
-define(LISTBOX_HEIGHT, 160).
-define(LISTBOX_XPOS, ?FRAME_XPOS-2).
-define(LISTBOX_YPOS, ?SMALL_WIN_HEIGHT+8).
-define(BTN_WIDTH, 80).
-define(BTN_HEIGHT, 30).
-define(BTN_XPOS, ?FRAME_WIDTH+2*?FRAME_XPOS+1).
-define(BG_COLOUR, {217,217,217}).






create_window(true) ->
    gs:config(win, [raise]);
create_window(false) ->
    gs:window(win, gs:start(), [{width,?WIN_WIDTH},
				{height,?SMALL_WIN_HEIGHT},
				{data,small},
				{bg,?BG_COLOUR},
				{title,"Search Object"},
				{destroy,true},
				{configure,true},
				{cursor,arrow}
			       ]),

    F = gs:frame(win, [{width,?FRAME_WIDTH},
		       {height,?FRAME_HEIGHT},
		       {x,?FRAME_XPOS},
		       {y,?FRAME_YPOS},
		       {bw,2},
		       {bg,?BG_COLOUR}
		      ]),

    gs:label(F, [{width,80},
		 {height,25},
		 {x,?ENTRY_XPOS},
		 {y,3},
		 {align,w},
		 {bg,?BG_COLOUR},
		 {fg, {0,0,0}},
		 {label, {text,"Search for:"}}
		]),

    gs:entry(entry, F, [{width,?ENTRY_WIDTH},
			{height,30},
			{x,?ENTRY_XPOS},
			{y,?ENTRY_YPOS},
			{insert, {0,"<Enter an Erlang term>"}},
			{bg, {255,255,255}},
			{fg, {0,0,0}},
			{cursor,text},
			{justify,left},
			{keypress,true},
			{setfocus,true}
		       ]),

    gs:button(search, win, [{width,?BTN_WIDTH},
			    {height,?BTN_HEIGHT},
			    {x,?BTN_XPOS},
			    {y,?FRAME_YPOS},
			    {label, {text,"Search"}},
			    {bg,?BG_COLOUR},
			    {fg, {0,0,0}}
			   ]),
    gs:button(cancel, win, [{width,?BTN_WIDTH},
			    {height,?BTN_HEIGHT},
			    {x,?BTN_XPOS},
			    {y,?FRAME_YPOS+?BTN_HEIGHT+10},
			    {label, {text,"Cancel"}},
			    {data,cancel},
			    {bg,?BG_COLOUR},
			    {fg, {0,0,0}}
			   ]),
    expand_window(),
    gs:config(entry, [{select, {0,1000}}]),
    gs:config(win, [{map,true}]).
    
			    


resize_window(false) ->
    done;
resize_window(true) ->
    gs:config(win, [{width,?WIN_WIDTH},
		    {height,?BIG_WIN_HEIGHT}
		   ]).
			    


		   
reset_window(false) ->
    done;
reset_window(true) ->
    gs:config(listbox, [clear]),
    gs:config(objects_found, [{label, {text,""}}]).


				

destroy_window(false) ->
    done;
destroy_window(true) ->
    gs:destroy(win).



mark_busy(false) ->
    done;
mark_busy(true) ->
    gs:config(win, [{cursor,busy}]),
    gs:config(entry, [{cursor,busy}]).
    



mark_nonbusy(false) ->
    done;
mark_nonbusy(true) ->
    gs:config(win, [{cursor,arrow}]),
    gs:config(entry, [{cursor,text}]).

	    


get_pattern_and_search(DbList) ->
    case gs:read(entry,text) of
	[] ->
	    gs:config(win, [beep]),
	    tool_utils:notify(win, ["Please enter a valid Erlang term!"]),
	    [];
	Pattern ->
	    gs:config(cancel, [{label, {text,"Stop"}}]),
	    mark_busy(true),
	    reset_window(true),
	    SearchRes = search(Pattern, DbList),
	    gs:config(cancel, [{label, {text,"Cancel"}}]),
	    mark_nonbusy(true),
	    SearchRes
    end.
	    



update_search(false, _DbList) ->
    done;
update_search(true, DbList) ->
    case gs:read(entry,text) of
	[] ->
	    [];
	Pattern ->
	    gs:config(cancel, [{label, {text,"Stop"}}]),
	    mark_busy(true),
	    reset_window(true),
	    SearchRes = search(Pattern, DbList),
	    gs:config(cancel, [{label, {text,"Cancel"}}]),
	    mark_nonbusy(true),
	    SearchRes
    end.
    




expand_window() ->
    gs:listbox(listbox, win, [{width,?LISTBOX_WIDTH},
			      {height,?LISTBOX_HEIGHT},
			      {x,?LISTBOX_XPOS},
			      {y,?LISTBOX_YPOS},
			      {bg, {255,255,255}},
			      {fg, {0,0,0}},
			      {scrollbg,?BG_COLOUR},
			      {scrollfg,?BG_COLOUR},
			      {hscroll,bottom},
			      {vscroll,right},
			      {click,true},
			      {doubleclick,false},
			      {selectmode,single}
			     ]),
    gs:label(objects_found, win, [{width,?LISTBOX_WIDTH},
				  {height,25},
				  {x,?LISTBOX_XPOS},
				  {y,?LISTBOX_YPOS+?LISTBOX_HEIGHT+12},
				  {align,w},
				  {bg,?BG_COLOUR},
				  {fg, {0,0,0}}
				 ]),
    gs:config(win, [{width,?WIN_WIDTH},
		    {height,?BIG_WIN_HEIGHT}
		   ]).
    


    



search(Pattern, List) ->
    traverse(Pattern, List, 1, length(List), []).


    



traverse(Pattern, [Object | T], Row, Length, Acc) ->
    NewAcc = case compare(Pattern, Object, lists:flatten(io_lib:write(Object))) of
		 found ->
		     RowStr    = integer_to_list(Row),
		     LengthStr = integer_to_list(Length),
		     gs:config(listbox, 
			       [{add,
				 "  Row " ++ RowStr ++ ":" ++
				 lists:duplicate(length(LengthStr)-length(RowStr), 
						 " ") ++
				 "   " ++ 
				 lists:flatten(io_lib:write(Object))}
			       ]),
		     gs:config(objects_found, 
			       [{label, 
				 {text,integer_to_list(length(Acc)+1) ++ 
				  " object(s) found"}}
			       ]),
		     [{Row,Object} | Acc];
		 not_found ->
		     Acc
	     end,
    receive 
	{gs,cancel,click,_Data,_Args} ->
	    gs:config(objects_found, 
		      [{label, 
			{text,integer_to_list(gs:read(listbox,size)) ++ 
			 " object(s) found"}}
		      ]),
	    lists:reverse(NewAcc)
    after 
	0 ->
	    traverse(Pattern, T, Row+1, Length, NewAcc)
    end;
traverse(_Pattern, [], _N, Length, Acc) ->
    gs:config(objects_found, 
	      [{label, 
		{text,integer_to_list(gs:read(listbox,size)) ++ 
		 " object(s) found"}}
	      ]),
    lists:reverse(Acc).






compare(Pattern, Elem, ElemStr) when constant(Elem), Pattern =/= ElemStr ->
    not_found;
compare(Pattern, Elem, Pattern) ->
       %% Even the case Pattern = "{}" or "[]"!!!
    found;
compare(Pattern, Elem, ElemStr) when list(Elem) ->
    traverse_list(Pattern, Elem);
compare(Pattern, Elem, ElemStr) when tuple(Elem) ->
    traverse_tuple(Pattern, Elem, 1, size(Elem)).
	




traverse_tuple(Pattern, Tuple, N, Stop) when N =< Stop ->
    Elem = element(N,Tuple),
    case compare(Pattern, Elem, lists:flatten(io_lib:write(Elem))) of
	found ->
	    found;
	not_found ->
	    traverse_tuple(Pattern, Tuple, N+1, Stop)
    end;
traverse_tuple(_Pattern, _Tuple, N, Stop) when N > Stop ->
    not_found.






traverse_list(Pattern, [H | T]) ->
    case compare(Pattern, H, lists:flatten(io_lib:write(H))) of
	found ->
	    found;
	not_found ->
	    traverse_list(Pattern, T)
    end;
traverse_list(_Pattern, []) ->
    not_found.

