%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%*********************************************************************
%%%
%%%   Filename:         %M%
%%%
%%%   Author:           EPK/TE  Fredrik Gustafson 
%%% 
%%%   SCCS Delta ID:    %I%
%%%
%%%   SCCS Delta Date:  %E%
%%%
%%%   SCCS ID:          @(#)  %M%  %I%  %E%
%%%
%%%   Description:      
%%%
%%%*********************************************************************
%%%
%%%   COPYRIGHT ERICSSON TELECOM AB 1998
%%%
%%%   THE COPYRIGHT TO THE COMPUTER PROGRAM HEREIN IS THE
%%%   PROPERTY OF ERICSSON TELECOM AB, SWEDEN. THE PROGRAM
%%%   MAY BE USED AND/OR COPIED ONLY WITH THE WRITTEN PERMISSION
%%%   FROM ERICSSON TELECOM AB, OR IN ACCORDANCE WITH THE TERMS
%%%   AND CONDITIONS STIPULATED IN THE AGREEMENT/CONTRACT UNDER
%%%   WHICH THE PROGRAM HAS BEEN SUPPLIED.
%%%
%%%*********************************************************************
%%%
%%%   Revision information:
%%%
%%%   Date      Name                       Description
%%%   ------    -----------------------    -----------
%%%
%%%
%%%*********************************************************************



-module(tv_ets_rpc).
-author('fredrik@erlang.ericsson.se').
-vsn('$Revision: /main/release/free/1').
-copyright('Copyright (C) 1991-98 Ericsson Telecom AB').



-export([all/1,
	 info/3,
	 tab2list/2
	]).




%%% There is a pathological case where the node gets distributed, meaning
%%% that the nodename given as argument to the functions below no longer is
%%% the nodename returned from the BIF node/0. If we then try to make an rpc-
%%% call to the original node it will appear as if the node is down.
%%% However, this case may only occur if the original node was non-distributed,
%%% meaning that the original nodename has to be 'nonode@nohost', should this
%%% strange case occur. This leads us to the solution to treat this nodename
%%% as a special case: we never compare this nodename to the current local nodename,
%%% instead we always treat it as the local node.


all('nonode@nohost') ->
    chk(catch ets:all());
all(Node) when Node == node() ->
    chk(catch ets:all());
all(Node) when node() == 'nonode@nohost' ->
    chk(catch ets:all());
all(Node) ->
    chk(catch rpc:block_call(Node, ets, all, [])).




info('nonode@nohost', TabId, What) ->
    chk(catch ets:info(TabId, What));
info(Node, TabId, What) when Node == node() ->
    chk(catch ets:info(TabId, What));
info(Node, TabId, What) when node() == 'nonode@nohost' ->
    chk(catch ets:info(TabId, What));
info(Node, TabId, What) ->
    chk(catch rpc:block_call(Node, ets, info, [TabId, What])).
    



tab2list('nonode@nohost', TabId) ->
    chk(catch ets:tab2list(TabId));
tab2list(Node, TabId) when Node == node() ->
    chk(catch ets:tab2list(TabId));
tab2list(Node, TabId) when node() == 'nonode@nohost' ->
    chk(catch ets:tab2list(TabId));
tab2list(Node, TabId) ->
    chk(catch rpc:call(Node, ets, tab2list, [TabId])).






chk(Result) ->
    case Result of
	undefined ->
	    throw(no_table);
	_Anything when list(Result) ->
	    Result;
	_Anything when atom(Result) ->
	    Result;
	_Anything when integer(Result) ->
	    Result;
	_Anything when pid(Result) ->
	    Result;

	%% Messages received when node is down.
	{badrpc, nodedown} ->
	    throw(nodedown);
	{'EXIT', nodedown} ->
	    throw(nodedown);

	%% Messages received when table doesn't exist.
	{'EXIT', {badarg, {ets,local_info,Args}}} ->  
	       %% Due to inconsistencies in R2D and earlier versions:
	       %% ets:info/1 returned 'undefined' when table didn't
	       %% exist, while ets:info/2 returned the exit-signal 
	       %% above. This was corrected in R3A - now both functions 
	       %% return 'undefined'  :-)
	    throw(no_table);
	{badrpc, {'EXIT', {badarg,Reason}}} ->
	    throw(no_table);
	{'EXIT', {badarg,Reason}} ->
	    throw(no_table);
	Error when tuple(Error) ->   
	    throw({unexpected_error,Error})
    end.
	
