%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%*********************************************************************
%%%
%%%   Filename:         tv_ip.erl
%%%
%%%   Author:           EPK/TE  Fredrik Gustafson 
%%% 
%%%   SCCS Delta ID:    1.2
%%%
%%%   SCCS Delta Date:  97/04/28
%%%
%%%   SCCS ID:          @(#)  tv_ip.erl  1.2  97/04/28
%%%
%%%   Description:      
%%%
%%%*********************************************************************
%%%
%%%   COPYRIGHT ERICSSON TELECOM AB 1997
%%%
%%%   THE COPYRIGHT TO THE COMPUTER PROGRAM HEREIN IS THE
%%%   PROPERTY OF ERICSSON TELECOM AB, SWEDEN. THE PROGRAM
%%%   MAY BE USED AND/OR COPIED ONLY WITH THE WRITTEN PERMISSION
%%%   FROM ERICSSON TELECOM AB, OR IN ACCORDANCE WITH THE TERMS
%%%   AND CONDITIONS STIPULATED IN THE AGREEMENT/CONTRACT UNDER
%%%   WHICH THE PROGRAM HAS BEEN SUPPLIED.
%%%
%%%*********************************************************************
%%%
%%%   Revision information:
%%%
%%%   Date      Name                       Description
%%%   ------    -----------------------    -----------
%%%
%%%
%%%*********************************************************************



-module(tv_ip).
-author('fredrik@erlang.ericsson.se').
-vsn('$Revision: /main/release/free/1').
-copyright('Copyright (C) 1991-97 Ericsson Telecom AB').



-export([ip/1]).



-include("tv_int_msg.hrl").


-define(NOF_LABELS, 25).

-define(DEFAULT_BG_COLOR, {217, 217, 217}).







%%%*********************************************************************
%%% EXTERNAL FUNCTIONS
%%%*********************************************************************



%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


ip(Master) ->
    W = gs:window(gs:start(), [{map, true},
			       {width, 302},
			       {height, 38},
			       {bg, ?DEFAULT_BG_COLOR},
			       {title, "Launching..."}
			      ]),
    C = gs:canvas(W, [{width, 40},
		      {height, 35},
		      {x, 0},
		      {bg, {255, 255, 255}}
		     ]),
    gs:create(image, C, [{load_gif, code:priv_dir(tv) ++ "/erlang.gif"}]),
    BgLabel = gs:label(W, [{width, 252},
			   {height, 12}, 
			   {x, 47},
			   {y, 23},
			   {bg, {0, 0, 0}},
			   {cursor, arrow}
			  ]),
    
    LabelList = create_labels(?NOF_LABELS, W, 48),
    
    L = gs:label(W, [{width, 250},
		     {height, 18},
		     {x, 47},
		     {y, 0},
		     {bg, ?DEFAULT_BG_COLOR},
		     {fg, {0, 0, 0}},
		     {align, w}
		    ]),
    loop(1, LabelList, L).









%%%*********************************************************************
%%% INTERNAL FUNCTIONS
%%%*********************************************************************




%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


create_labels(0, _WinId, _Xpos) ->
    [];
create_labels(N, WinId, Xpos) ->
    Width = 10,
    Xdiff = Width,
    LabelId = gs:label(WinId, [{width, Width},
			       {height, 10}, 
			       {x, Xpos},
			       {y, 24},
			       {bg, {235, 235, 235}},
			       {cursor, arrow}
			      ]),

    [LabelId | create_labels(N - 1, WinId, Xpos + Xdiff)].











%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


loop(N, LabelList, L) ->
    receive
	Msg ->
	    case Msg of

		#ip_update{nof_elements_to_mark = X, text = Text} ->
		    update_window(LabelList, N, N + X, L, Text),
		    loop(N + X, LabelList, L);

		#ip_quit{} ->
		    update_labels(LabelList, N, ?NOF_LABELS),
		    receive
		    after 1000 ->
			    done
		    end,
		    done;

		_Other ->
		    loop(N, LabelList, L)
	    end
    end.











%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


update_window(LabelList, N, Hi, LblId, Text) ->
    gs:config(LblId, [{label, {text, Text}}]),
    update_labels(LabelList, N, Hi).








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


update_labels(LabelList, N, Hi) when N > ?NOF_LABELS ->
    done;
update_labels(LabelList, N, Hi) when N >= Hi ->
    done;
update_labels(LabelList, N, Hi) ->
    LabelId = lists:nth(N, LabelList),
    gs:config(LabelId, [{bg, {0, 0, 255}}]),
    update_labels(LabelList, N + 1, Hi).
    











