%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%*********************************************************************
%%%
%%%   Filename:         %M%
%%%
%%%   Author:           EPK/TE  Fredrik Gustafson 
%%% 
%%%   SCCS Delta ID:    %I%
%%%
%%%   SCCS Delta Date:  %E%
%%%
%%%   SCCS ID:          @(#)  %M%  %I%  %E%
%%%
%%%   Description:      
%%%
%%%*********************************************************************
%%%
%%%   COPYRIGHT ERICSSON TELECOM AB 1998
%%%
%%%   THE COPYRIGHT TO THE COMPUTER PROGRAM HEREIN IS THE
%%%   PROPERTY OF ERICSSON TELECOM AB, SWEDEN. THE PROGRAM
%%%   MAY BE USED AND/OR COPIED ONLY WITH THE WRITTEN PERMISSION
%%%   FROM ERICSSON TELECOM AB, OR IN ACCORDANCE WITH THE TERMS
%%%   AND CONDITIONS STIPULATED IN THE AGREEMENT/CONTRACT UNDER
%%%   WHICH THE PROGRAM HAS BEEN SUPPLIED.
%%%
%%%*********************************************************************
%%%
%%%   Revision information:
%%%
%%%   Date      Name                       Description
%%%   ------    -----------------------    -----------
%%%
%%%
%%%*********************************************************************



-module(tv_mnesia_rpc).
-author('fredrik@erlang.ericsson.se').
-vsn('$Revision: /main/release/free/1').
-copyright('Copyright (C) 1991-98 Ericsson Telecom AB').



-export([%match_object/2,
	 system_info/2,
	 table_info/3,
	 transaction/2
	]).





%%% There is a pathological case where the node gets distributed, meaning
%%% that the nodename given as argument to the functions below no longer is
%%% the nodename returned from the BIF node/0. If we then try to make an rpc-
%%% call to the original node it will appear as if the node is down.
%%% However, this case may only occur if the original node was non-distributed,
%%% meaning that the original nodename has to be 'nonode@nohost', should this
%%% strange case occur. This leads us to the solution to treat this nodename
%%% as a special case: we never compare this nodename to the current local nodename,
%%% instead we always treat it as the local node.




%match_object(Node, Pattern) when Node == node() ->
%    mnesia:match_object(Pattern);
%match_object(Node, Pattern)   ->
%    rpc:block_call(Node, mnesia, match_object, [Pattern]).




system_info('nonode@nohost', Key) ->
    chk(catch mnesia:system_info(Key));
system_info(Node, Key) when Node == node() ->
    chk(catch mnesia:system_info(Key));
system_info(Node, Key) when node() == 'nonode@nohost' ->
    chk(catch mnesia:system_info(Key));
system_info(Node, Key) ->
    chk(catch rpc:block_call(Node, mnesia, system_info, [Key])).




table_info('nonode@nohost', Tab, Item) ->
    chk(catch mnesia:table_info(Tab, Item));
table_info(Node, Tab, Item) when Node == node() ->
    chk(catch mnesia:table_info(Tab, Item));
table_info(Node, Tab, Item) when node() == 'nonode@nohost' ->
    chk(catch mnesia:table_info(Tab, Item));
table_info(Node, Tab, Item) ->
    chk(catch rpc:block_call(Node, mnesia, table_info, [Tab, Item])).




transaction('nonode@nohost', Fun) ->
    chk(catch mnesia:transaction(Fun));
transaction(Node, Fun) when Node == node() ->
    chk(catch mnesia:transaction(Fun));
transaction(Node, Fun) when node() == 'nonode@nohost' ->
    chk(catch mnesia:transaction(Fun));
transaction(Node, Fun) ->
    chk(catch rpc:block_call(Node, mnesia, transaction, [Fun])).




chk(Result) ->
    case Result of
	_Anything when list(Result) ->
	    Result;
	_Anything when atom(Result) ->
	    Result;
	_Anything when integer(Result) ->
	    Result;
	_Anything when pid(Result) ->
	    Result;

	{badrpc,nodedown} ->
	    throw(nodedown);
	{'EXIT', nodedown} ->
	    throw(nodedown);

	{'EXIT', {aborted, {node_not_running, Node}}} ->
	    throw(mnesia_not_started);
	{'EXIT', {'EXIT', {aborted, {node_not_running,Node}}}} ->
	    throw(mnesia_not_started);
	{badrpc, {'EXIT', {aborted, {node_not_running,Node}}}} ->
	    throw(mnesia_not_started);
	{badrpc, {'EXIT', {aborted, {no_exists,Table,Args}}}} ->
	    throw(mnesia_not_started);
	{badrpc, Reason} ->
	    throw(mnesia_not_started);
	{'EXIT', {undef, {mnesia,Fcn,Args}}} ->
	    throw(mnesia_not_started);
	
	{'EXIT', Reason} ->
	    throw({unexpected_error, Reason});
	
	Other when tuple(Other) ->
	       %% For example wild_pattern requests return a tuple!
	    Other;
	
	Other ->
	    io:format("Unexpected return value: ~p~n", [Other])
    end.
	

