%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%*********************************************************************
%%%
%%%   Filename:         tv_pc_menu_handling.erl
%%%
%%%   Author:           EPK/TE  Fredrik Gustafson 
%%% 
%%%   SCCS Delta ID:    1.3
%%%
%%%   SCCS Delta Date:  97/06/12
%%%
%%%   SCCS ID:          @(#)  tv_pc_menu_handling.erl  1.3  97/06/12
%%%
%%%   Description:      Part of pc handling the creation of menus, as well as 
%%%                     treating the signals these menus results in, when chosen.
%%%
%%%*********************************************************************
%%%
%%%   COPYRIGHT ERICSSON TELECOM AB 1998
%%%
%%%   THE COPYRIGHT TO THE COMPUTER PROGRAM HEREIN IS THE
%%%   PROPERTY OF ERICSSON TELECOM AB, SWEDEN. THE PROGRAM
%%%   MAY BE USED AND/OR COPIED ONLY WITH THE WRITTEN PERMISSION
%%%   FROM ERICSSON TELECOM AB, OR IN ACCORDANCE WITH THE TERMS
%%%   AND CONDITIONS STIPULATED IN THE AGREEMENT/CONTRACT UNDER
%%%   WHICH THE PROGRAM HAS BEEN SUPPLIED.
%%%
%%%*********************************************************************
%%%
%%%   Revision information:
%%%
%%%   Date      Name                       Description
%%%   ------    -----------------------    -----------
%%%   970427    EPK/TE  F Gustafson        First version.
%%%
%%%
%%%*********************************************************************



-module(tv_pc_menu_handling).
-author('fredrik@erlang.ericsson.se').
-vsn('$Revision: /main/release/free/1').
-copyright('Copyright (C) 1991-98 Ericsson Telecom AB').



-export([create_menus/1, 
	 new_tv/1,
	 exit_button/1, 
	 search_object/1,
	 open_table/1, 
	 set_poll_interval/1, 
	 poll_table/1, 
	 sort_rising_order/1, 
	 sort_falling_order/1, 
	 no_sorting/1,
	 table_info/1, 
	 help_button/1,
	 get_window_title/5]).





-include("tv_int_def.hrl").
-include("tv_int_msg.hrl").
-include("tv_pc_int_def.hrl").








%%%*********************************************************************
%%% EXTERNAL FUNCTIONS
%%%*********************************************************************




%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%
%%======================================================================
%%
%%    Shortcuts currently used, in alphabetical order:
%% 
%%    f  ->  "Sort Falling Order"
%%    h  ->  "Help"
%%    i  ->  "Table Info"
%%    n  ->  "No Sorting"
%%    o  ->  "Open Table"
%%    p  ->  "Poll Table"
%%    r  ->  "Sort Rising Order"
%%    s  ->  "Search Object"
%%    v  ->  "Set Poll Interval"
%%    x  ->  "Exit"


create_menus(PwPid) ->
       % Order pw to create the 'File' menu.
    ?GRAPH_FUNC_FILE:create_menu(PwPid,
				 " File ", 
				 1,
				 [{" Open Table... ", open_table, 1, o},
				  separator,
				  {" New Table Visualizer ", new_tv, 1, no_char},
				  separator,
				  {" Exit ", exit_button, 2, x}
				 ]),
       % Order pw to create the 'Tools' menu.
    ?GRAPH_FUNC_FILE:create_menu(PwPid,
				 " Tools ", 
				 1,
				 [{" Search Object ", search_object, 1, s},
				  separator,
				  {" Sort Rising Order ", sort_rising_order, 6, no_char},
				  {" Sort Falling Order ", sort_falling_order, 6, no_char},
				  {" No Sorting ", no_sorting, 1, no_char}
				 ]),
       % Order pw to create the 'Options' menu.
    ?GRAPH_FUNC_FILE:create_menu(PwPid,
				 " Options ", 
				 1,
				 [{" Poll Table ", poll_table, 1, p},
				  {" Set Poll Interval... ", set_poll_interval, 1, no_char}
				 ]),
       % Order pw to create the 'View' menu.
    ?GRAPH_FUNC_FILE:create_menu(PwPid,
				 " View ", 
				 1,
				 [{" Table Info ", table_info, 1, i}
				 ]).

    





%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


new_tv(ProcVars) ->
    tv:start(),
    ProcVars.





%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


exit_button(_ProcVars) ->
    exit(normal).










%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


help_button(ProcVars) ->
    HelpFile = filename:join(code:priv_dir(tv), "../doc/index.html"),
    tool_utils:open_help(gs:start([{kernel, true}]), HelpFile),
    ProcVars.

    







%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


table_info(ProcVars) ->
    #process_variables{table_id    = TableId, 
		       etsread_pid = EtsreadPid} = ProcVars,
    
    case TableId of 
	undefined ->
	    done;
	_OtherValue ->
	    EtsreadPid ! #etsread_show_table_info{sender = self()}
    end,
    ProcVars.








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


sort_rising_order(ProcVars) ->
    request_sort_settings(ProcVars#process_variables.pd_pid, true, false), 
    ProcVars.
    




    



%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


sort_falling_order(ProcVars) ->
    request_sort_settings(ProcVars#process_variables.pd_pid, true, true), 
    ProcVars.









%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


no_sorting(ProcVars) ->
    request_sort_settings(ProcVars#process_variables.pd_pid, false, false), 
    ProcVars.    








%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


set_poll_interval(ProcVars) ->
    #process_variables{etsread_pid   = EtsreadPid,
		       poll_interval = PollInterval}  = ProcVars,
    
    case tv_poll_dialog:start(PollInterval) of
	cancel ->
	    ProcVars;
	NewPollInterval ->
	    EtsreadPid ! #etsread_set_poll_interval{sender   = self(),
						    interval = NewPollInterval},
	    ProcVars#process_variables{poll_interval = NewPollInterval}
    end.
	    
	    
    





%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


poll_table(ProcVars) ->
    EtsreadPid = ProcVars#process_variables.etsread_pid,
    EtsreadPid ! #etsread_poll_table{sender = self()},
    ProcVars.
    
    






%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


search_object(ProcVars) ->
    DbsPid = ProcVars#process_variables.dbs_pid,
    DbsPid ! #dbs_search_req{sender=self()},
    ProcVars.





open_table(ProcVars) ->
    #process_variables{current_node = CurrNode,
		       table_type   = CurrTableType}  = ProcVars,
    
    case tv_tab_dialog:start(CurrTableType, CurrNode) of
	cancel ->
	    ProcVars;
	{NewCurrNode, TableId, NewTableType, TableName, TableNumber} ->
	    #process_variables{dbs_pid       = DbsPid,
			       etsread_pid   = EtsreadPid,
			       pw_pid        = PwPid,
			       pd_pid        = PdPid,
			       poll_interval = PollInterval}  = ProcVars,

	    PdPid ! #pd_new_table{sender = self()},

	    WinTitle = get_window_title(NewCurrNode, TableId, NewTableType,
					TableName, TableNumber),

	    PwPid ! #pw_set_window_title{sender = self(),
					 win_title = WinTitle},

	    {Type, KeyPos} = init_etsread(EtsreadPid, DbsPid, NewCurrNode, TableId, 
					  NewTableType, PollInterval),
	    init_dbs(DbsPid, Type, KeyPos),
	    ProcVars#process_variables{current_node = NewCurrNode,
				       table_id     = TableId,
				       table_type   = NewTableType,
				       table_name   = TableName,
				       table_number = TableNumber}
    end.









get_window_title(Node, TableId, TableType, TableName, TableNumber) ->
    TableNameStr   = lists:flatten(io_lib:write(TableName)),
    TableNumberStr = lists:flatten(io_lib:write(TableNumber)),
    WinTitleSuffix = "         Node:  " ++ atom_to_list(Node),
    case TableType of 
	mnesia ->
	    "Mnesia Table:  " ++ TableNameStr ++ WinTitleSuffix;
	ets ->
	    case TableNumber of
		undefined ->
		    "ETS Table:  " ++ TableNameStr ++ WinTitleSuffix;
		AnyNumber ->
		    "ETS Table:  " ++ TableNameStr ++ "  #" ++ TableNumberStr ++ WinTitleSuffix
	    end
    end.





%%%*********************************************************************
%%% INTERNAL FUNCTIONS
%%%*********************************************************************



%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


init_etsread(EtsreadPid, DbsPid, Node, TableId, TableType, PollInterval) ->
    EtsreadPid ! #etsread_deblock{sender        = self(),
				  dbs_pid       = DbsPid,
				  node          = Node,
				  table_id      = TableId,
				  table_type    = TableType, 
				  poll_interval = PollInterval
				 },
    receive
	#etsread_deblock_cfm{type = Type, keypos = KeyPos} ->
	    {Type, KeyPos}
    after 10000 ->
	    exit(error)
    end.

	





%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


init_dbs(DbsPid, Type, KeyPos) ->
    DbsPid ! #dbs_deblock{sender = self(),
			  type   = Type,
			  keypos = KeyPos,
			  sublist_length = ?ITEMS_TO_DISPLAY
			 },
    receive
	#dbs_deblock_cfm{} ->
	    done
    after 10000 ->
	    exit(error)
    end.









%%======================================================================
%% Function:      
%%
%% Return Value:  
%%
%% Description:   
%%
%% Parameters:    
%%======================================================================


request_sort_settings(PdPid, Sorting, Reverse) ->
    PdPid ! #pd_get_sort_settings{sender  = self(),
				  sorting = Sorting,
				  reverse = Reverse
				 }.
	    
	





