%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%% File    : exref_subgraph.erl
%%% Author  : Klas Eriksson <eklas@geir>
%%% Purpose : Add on to exref to find subgraphs
%%% Created : 30 Jul 1996 by Klas Eriksson <eklas@geir>
%%% $Id: exref_subgraph.erl,v 1.1 1996/11/28 08:51:58 chris Exp $
%%%
-module(exref_subgraph).
-copyright('Copyright (c) 1991-97 Ericsson Telecom AB').
-vsn('$Revision: /main/release/free/1').
-author('eklas@geir').

-compile(export_all).
%%-export([Function/Arity, ...]).

%% usage: exref:analyse(user_defined, {exref_subgraph,func_nodes, Arg}).
%% Arg: [MaxDistance|infinity,Base]
%% MaxDistance:integer() 
%% Base: list of {Mod,Func,Arity}          

func_nodes(G,Level,Front) ->
    BeenAt = [],
    Edges = digraph:edges(G),
    R = deepNearBy(G, Level, Edges, Front, BeenAt).

func_nodes_and_edges(G,Level,Front) ->
    Funcs=func_nodes(G,Level,Front),
    {Funcs,edges(digraph:edges(G),G,Funcs)}.

deepNearBy(G, 0, Edges, Front, Res) -> lists:append(Res,Front);
deepNearBy(G, Level, Edges, [], Res) -> Res;
deepNearBy(G, Level, Edges, Front, BeenAt) ->
    AllBeenAt = lists:append(Front, BeenAt),
    NewFront = expandFrontOne(G, Edges, Front, AllBeenAt, []),
    deepNearBy(G, decr(Level), Edges, NewFront, AllBeenAt).

expandFrontOne(G, [], F, BeenAt, Res) -> Res;
expandFrontOne(G, [EdgeId | Edges],
	       Front, BeenAt, AccumulatedFront) ->
    Edge = digraph:edge(G, EdgeId),
    {_EdgeId, CallingFunc, CalledFunc, _} = Edge,
    case lists:member(CallingFunc, Front) of
	false ->
	    expandFrontOne(G, Edges, Front, BeenAt, AccumulatedFront);
	true ->
	    case lists:member(CalledFunc, BeenAt) of
		true ->
		    expandFrontOne(G, Edges, Front, BeenAt, AccumulatedFront);
		false ->
		    case lists:member(CalledFunc, AccumulatedFront) of
			true ->
			    expandFrontOne(G, Edges, Front,
					   BeenAt, AccumulatedFront);
			false ->
			    expandFrontOne(G, Edges, Front, BeenAt,
					   [CalledFunc | AccumulatedFront])
		    end
	    end
    end.

edges([],_G,Funcs) ->
    [];
edges([Eid|Es],G,Funcs) ->
    Edge = digraph:edge(G, Eid),
    {_EdgeId, CallingFunc, CalledFunc, _} = Edge,
    case two_members(CallingFunc,CalledFunc,Funcs) of
	true -> [Edge|edges(Es,G,Funcs)];
	false -> edges(Es,G,Funcs)
    end.

two_members(A,B,List) ->
    two_members(A,B,List,false,false).

two_members(A,B,[A|T],_,false) -> two_members(A,B,T,true,false);
two_members(A,B,[A|T],_,true) -> true;
two_members(A,B,[B|T],false,_) -> two_members(A,B,T,false,true);
two_members(A,B,[B|T],true,_) -> true;
two_members(A,B,[_|T],Abool,Bbool) -> two_members(A,B,T,Abool,Bbool);
two_members(A,B,[],_,_) -> false.

decr(infinity) -> infinity;
decr(N) -> N-1.
