%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%% ------------------------------------------------------------
%% File Selection Dialog
%% Written by Ola Samuelsson in July 1995
%%  Ericsson Infocom Consultants AB / Erlang Systems Division
%% ------------------------------------------------------------
%% $Id: file_dial.erl,v 1.2 1996/11/28 09:42:32 chris Exp chris $
%%
-module(file_dial).
-copyright('Copyright (c) 1991-97 Ericsson Telecom AB').
-vsn('$Revision: /main/release/free/1').
%%-revision('$Revision: 1.2 $').
-author('ola@erlang.ericsson.se').
-modified('$Date: 1996/11/28 09:42:32 $').  %'
-modified_by('chris@erlang.ericsson.se').

-export([refresh/1, fileloop/2]).

-include_lib("kernel/include/file.hrl").

-define(Extension, ".erl").
-define(Filter_extension, {extension, ?Extension}).
-define(Root, "/").
-define(Single_dot, "./").
-define(Double_dot, "../").
-define(No_file, []).

%% ------------------------------------------------------------

fileloop(InDir, Modlb_show) ->
    Dir = filename:join([InDir]),     % Make slashes go "/" and remove unneeded "/"
    receive
	{gs,load,click,_,_} ->
	    ok = gs:config(win,{cursor,busy}),
	    New_modlb_show = load_name(Dir, Modlb_show),
	    ok = gs:config(win,{cursor,arrow}),
	    fileloop(Dir, New_modlb_show);
	{gs, fileentry, enter, _, _} ->
	    New_dir = fileentry_loop(Dir, true),
	    fileloop(New_dir, Modlb_show);	    
	{gs,filelb,click,_,_} ->
	    clicked(Dir, Modlb_show);
	{gs,filelb,doubleclick,_,[Index,Text|_]} ->
	    double_clicked(Dir, Index, Text, Modlb_show);
	{gs,fileframe,leave,_,_} ->
	    xref:main_loop(Dir, Modlb_show);
	X ->
	    fileloop(Dir, Modlb_show)
    end.

%% true and false is really changed or not changed

fileentry_loop(Dir, Is_dir_ok) ->
    receive
 	{gs, fileentry, keypress, _, ['Return'|_]} ->
	    case change_dir(Dir, Is_dir_ok) of
		{ok, New_dir} ->
		    fileentry_loop(New_dir, true);
		{not_ok, New_dir} ->
		    fileentry_loop(Dir, false)
	    end;
	{gs,fileentry,keypress,_,[Keysym|_]} ->
	    fileentry_loop(Dir, false);
	{gs, fileentry, leave, _, _} ->
	    case change_dir(Dir, Is_dir_ok) of
		{ok, New_dir} ->
		    ok = gs:config(fileentry, {text, add_slash(New_dir)}),
		    ok = gs:config(fileentry, {setfocus, false}),
		    New_dir;
		{not_ok, _} ->
		    ok = gs:config(fileentry, {text, add_slash(Dir)}),
		    ok = gs:config(fileentry, {setfocus, false}),
		    change_dir(Dir, false),
		    Dir
	    end;
	Other ->
	    fileentry_loop(Dir, Is_dir_ok)
    end.


change_dir(Dir, true)  ->
    {ok, Dir};
change_dir(Dir, false) ->
    %% Get a filename and normalize it
    case gs:read(fileentry,text) of
	?No_file ->				
	    New_dir = filename:absname(?Root);
	TmpDir ->
	    New_dir = filename:absname(TmpDir)
    end,

    case file:list_dir(New_dir) of
	{ok, _} ->
	    Items = refresh(New_dir),
	    ok = gs:config(filelb, {items, Items}),
	    {ok, New_dir};
	{error, ErrorString} ->
	    ok = gs:config(fileentry, {text, ErrorString}),
	    timer:sleep(1000),
	    ok = gs:config(fileentry, {text, add_slash(New_dir)}),
	    {not_ok, New_dir}
    end.

    
refresh(Dir) ->
    ok = gs:config(filelb,clear),
    ok = gs:config(fileentry,{text, add_slash(Dir)}),
    Items = [?Single_dot, ?Double_dot | get_files(Dir)],
    ok = gs:config(filelb, {items,Items}),
    Items.


%% load_name/1       (Block of functions)
%%
%% Purpose:   Analysis (of exref) the selected modules and/or the modules in a
%%            selected directory and put the result (on the screen) into
%%            the module-listbox (modlb).
%%
%% Help func: update_module_listbox/4 flatten_into_lists/1 get_mod_name/1
%%            load_into_exref/2       
%%
%% Returns: same as xref_gp:disp_mods/0, i.e.
%%          { [Def_module_names], [Lib_names], [Undefined_module_names] }  
%%
%%

load_name(Dir, Modlb_show) ->
    case gs:read(filelb, selection) of
	[] -> %% None selected
	    Modlb_show;
	Indices ->
	    %% List of integers corresponding to all selected names in 
	    %% file-listbox.
	    %%
	    %% Now we have to perform the following steps:
	    %%
	    %% 1. Pick all files and/or directories that is selected in 
	    %%    file-listbox.
	    %% 2. Out of 1. extract all modulenames out of filenames and files
	    %%    contained in a dir.
	    %% 3. Load all modulenames into EXREF.
	    %% 4. Perform the EXREF-analysis.
	    %% 5. Show modules in the module-listbox.
	    %% 6. Highlight the new modules together with those that were 
	    %%    highlighted before in the module-listbox.
	    %% 7. Remove highlight in file-listbox.
	    %% 8. Return all modules, contained in the module-listbox, 
	    %%    divided in the categories the_loaded_modules, std_modules, 
	    %%    undef_modules.
	    %%
	    Files_Dirs = lists:map(fun get_file_or_dir/2, [Dir], Indices),
	    Non_flat   = lists:foldl(fun get_mod_names/2, [], Files_Dirs),

	    case Non_flat of
		[] ->
		    Modlb_show;
		List ->

		    Newly_selected_mod_names =
			lists:map(fun parse/1, flatten_into_lists(Non_flat)),

		    load_into_exref(Dir, Files_Dirs),
		    {undefined_functions, Modules, Undefined_modules} =
			exref:analyse(undefined_functions),
		    {Def_mod, Lib_mod, Undef_mod} =
			xref_gp:divide_in_mod_types(Modules, Undefined_modules),
		    New_modlb_show = mod_listbox:update(Def_mod ++ Lib_mod,
							Undef_mod,
							Newly_selected_mod_names,
							Modlb_show),
		    rb_mod:set_greyscale_fg(New_modlb_show),
		    ok = gs:config(filelb, {selection, clear}),
		    set_funclb(Newly_selected_mod_names),
		    New_modlb_show
	    end
    end.

%% parse 
%%
parse(Txt) ->
    String = Txt ++ ". ",

    Term = case erl_scan:string(String) of
	       {ok,Ts,_} -> Ts;
	       ScanError -> exit({?MODULE, parse, Txt,
				  {erl_scan, string,
				   String,
				   ScanError}})
	   end,
    Name = case erl_parse:parse_term(Term) of
	       {ok,PName} -> PName;
	       ParseError -> exit({?MODULE, parse, Txt,
				   {erl_parse, parse_term,
				    String,
				    ParseError}})
	   end,
    Name.

set_funclb([]) -> true;
set_funclb([Top_name|_]) ->
    Name = atom_to_list(Top_name),
    ok = gs:config(funcentry, {label, {text, Name}}),
    xref_gp:disp_funcs(Name).


%% get_file_or_dir/2
%%
%% Returns: {dir, "abs_path_dir_name"} | {file, "file_name"} | not_a_valid_file
%%          where the names corresponds to an Index, in a graphical listbox,
%%          sent as argument. not_a_valid_file is returned if the file no 
%%          longer exists, or if it is not a proper file/directory.
%%

get_file_or_dir(Index, Dir) ->
    case gs:read(filelb,{get,Index}) of
	?Single_dot ->
	    {dir, Dir};
	?Double_dot ->
	    Newdir = up_one_dir(Dir),
	    {dir, Newdir};
	Name ->
	    AbsName = filename:join(Dir,Name),
	    case file:read_file_info(AbsName) of
		{ok, FileInfo} ->
		    case FileInfo#file_info.type of
			directory ->
			    {dir, AbsName};
			regular ->
			    {file, AbsName};
			Other ->
			    not_a_valid_file
		    end;
		{error, Reason} ->
		    not_a_valid_file
	    end
    end.


%% get_mod_name/1
%%
%% Returns: "module_name" | ["module_name_1",...,"module_name_n"]
%%          where the names in the list is contained in the directory.
%%           

get_mod_name({file, File_name}) ->
    filename:basename(File_name, ?Extension);
get_mod_name({dir, Dir_name}) ->
    case list_dir(Dir_name, ?Filter_extension) of
	{error, Reason } ->
	    error_logger:error_msg("M:file_dial F:get_mod_name/1 : "
				   "Error: ~w for ~w~n",
				   [Reason,
				    list_to_atom(Dir_name)]),
	    [];
	{_, File_listing } ->
	    [filename:basename(X, ?Extension) || X <- File_listing]
    end.
	    
%% get_mod_name/1
%%
%% Returns: "module_name" | ["module_name_1",...,"module_name_n"]
%%          where the names in the list is contained in the directory.
%%           

get_mod_names(not_a_valid_file, Acc) ->
    Acc;
get_mod_names({file, File_name},Acc) ->
    [filename:basename(File_name, ?Extension) | Acc];
get_mod_names({dir, Dir_name},Acc) ->
    case list_dir(Dir_name, ?Filter_extension) of
	{error, Reason } ->
	    error_logger:error_msg("M:file_dial F:get_mod_name/1 : "
				   "Error: ~w for ~w~n",
				   [Reason,
				    list_to_atom(Dir_name)]),
	    [];
	{_, File_listing } ->
	    lists:append([filename:basename(X, ?Extension) || X<-File_listing], Acc)

    end.
	    


%% flatten_into_lists/1
%%
%% Purpose: To flatten a list in which a list element with
%%          homogeneous elements is considered as an atomic
%%          element. An atomic list will not be processed any
%%          further.
%% Note ! : lists:flatten(["the", ["truth", "is", "out"], "there"])
%%                --> "thetruthisoutthere"
%%          flatten_into_lists(["the", ["truth", "is", "out"], "there"])
%%                --> ["the", "truth", "is", "out", "there"]
%%
%% Returns: [ "smalest_element_1",..., "smalest_element_n"]
%%
%% Arg:     a non flattened list.
%%

flatten_into_lists([]) ->
    [];
flatten_into_lists([H|T]) ->
    Is_atomic_list_element = fun (Element) when list(Element) ->
				     false;
				 (_) -> true
			     end,
    case lists:all(Is_atomic_list_element, H) of
	false ->
	    flatten_into_lists(H) ++ flatten_into_lists(T);
	true ->
	    [H | flatten_into_lists(T)]
    end.

%% load_into_exref/2
%%
%% Purpose: Load all files and/or files contained in a directory,
%%          into EXREF.
%%
%% Arg:     Dir = current directory
%%          Files_Dirs = list with elements {file, "filename"} |
%%                                          {dir, "dir_name"}
%%

load_into_exref(Dir, Files_Dirs) ->
    Load_file_or_dir =
	fun ({file, File}) ->
		Module = filename:basename(File, ?Extension),
		exref:directory_module(Dir, list_to_atom(Module));
	    ({dir, Directory}) ->
		exref:directory(Directory);
	    (not_a_valid_file) ->
		ignore
        end,
    lists:foreach(Load_file_or_dir, Files_Dirs).


%%
%% load_name/1   (End of block)


clicked(Dir, Modlb_show) ->
    [Idx|_] = gs:read(filelb,selection),
    File = gs:read(filelb,{get,Idx}),
    fileloop(Dir, Modlb_show).

%% double_clicked/4  (Block of functions)
%%
%% Purpose: Change and show the current directory in the
%%          file-frame.
%%

double_clicked(Dir, Index, Text, Modlb_show) ->
    case file:read_file_info(filename:join(Dir, Text)) of
	{ok, FileInfo} ->
	    case FileInfo#file_info.type of
		directory ->	    % down/up a dir (we added the slash so it is ok)
		    dc_dir(Dir, Text, Modlb_show);
		Other ->            % done
		    fileloop(Dir, Modlb_show)
	    end;
	{error, Reason} ->
	    exit(Reason)
    end.

dc_dir(Dir, Dirname, Modlb_show) ->
    ok = gs:config(win,{cursor,busy}),
    The_dir = case Dirname of
		  ?Single_dot ->
		      Dir;
		  ?Double_dot ->       % up one dir
		      up_one_dir(Dir);
		  A_dir ->             % down a dir
		      filename:join(Dir,A_dir)
	      end,
    refresh(The_dir),
    ok = gs:config(win,{cursor,arrow}),
    fileloop(The_dir, Modlb_show).
%%
%% double_clicked/4  (End of block)


get_files(Dir) ->
    {Dirs, Files} = list_dir(Dir, ?Filter_extension),
    lists:map(fun(A) -> A ++ "/" end, Dirs) ++ Files.

%% list_dir/2  (Block of functions) 
%%
%% Help
%% Functions: pick_dir/1 pick_files/2 is_file_of_interest/2
%%
%% Returns  : { ["dir_name_1",..., "dir_name_n"],
%%              ["file_name_1",..., "file_name_m"] } | {error, Reason}
%%            where all
%%            - file and dir names is contained in the directory specified 
%%              as argument.
%%            - namelists are sorted.
%%            - filenames fullfills the options specified as argument.
%%
%% Option  : {extension, ".extension_name"} | {noextension}
%%
%%           specify a certain property the files got to have
%%           to become part of the final result.
%% 

list_dir(Dir, Option) ->
    case file:list_dir(Dir) of
	{ok, Files} ->
	    Alldir = pick_dir(Dir, Files),
	    Allfiles = pick_files(Dir, Files, Option),
	    { lists:sort(Alldir), lists:sort(Allfiles) };
	{error, Reason} ->
	    error_logger:error_msg("M:file_dial F:list_dir/2 : "
				   "Error: ~w for ~w~n",
				   [Reason, list_to_atom(Dir)]),
	    {error, Reason}
    end.

%% pick_dir/1
%%
%% Returns : a list with all directory names contained in
%%           the list Files. If there is an error reading
%%           a certain file/dir.-name that filename will never
%%           appear in the returned list.
%%

pick_dir(Dir, Files) ->
    Select =
	fun(File) ->
		Abs_file = filename:join(Dir, File),
		case file:read_file_info(Abs_file) of
		    {ok, FileInfo} ->
			case FileInfo#file_info.type of
			    directory ->
				true;
			    Other ->
				false
			end;
		    {error, Reason} ->
			error_logger:error_msg("M:file_dial F:pick_dir/1 : "
					       "Error:~w on ~w~n", 
					       [Reason,
						list_to_atom(File)]),
			false
		end
	end,
    lists:filter(Select, Files).


%% pick_files/2
%%
%% Returns : a list with all filenames contained in
%%           the list Files. If there is an error occur reading
%%           a certain filename that filename will never
%%           appear in the returned list.
%%

pick_files(Dir, Files, Option) ->
    Select =
	fun(File) ->
		Abs_file = filename:join(Dir,File),
		case file:read_file_info(Abs_file) of
		    {ok, FileInfo} ->
			case FileInfo#file_info.type of
			    regular ->
				is_file_of_interest(File, Option);
			    _Other ->
				false
			end;
		    {error, Reason} ->
			error_logger:error_msg("M:file_dial F:pick_files/2 : "
					       "Error:~w on ~w~n",
					       [Reason,
						list_to_atom(File)]),
			false
		end
	end,
    lists:filter(Select, Files).

is_file_of_interest(File, {extension, Extension}) ->
    filename:extension(File) == Extension;
is_file_of_interest(File, {noextension}) ->
    true.

%%
%% list_dir/2  (End of block)


top_dir(AbsoluteDir) ->
    case filename:split(AbsoluteDir) of
	[_] ->
	    yes;				% Drive letter or "/" alone
	_ ->
	    no
    end.

up_one_dir(AbsoluteDir) ->
    case top_dir(AbsoluteDir) of
	yes ->
	    AbsoluteDir;
	no ->
	    DirParts = filename:split(AbsoluteDir),
	    ParDirParts = lists:sublist(DirParts,length(DirParts) - 1), % Remove last element
	    filename:join(ParDirParts)
    end.


add_slash(Path) ->
    case lists:last(Path) of
	$/ ->
	    Path;
	_ ->
	    Path ++ "/"
    end.

%% ----------------------------------------
%% done
