%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%----------------------------------------------------------------------
%%% File    : mod_listbox.erl
%%% Author  : Christofer Toernkvist <chris@jones>
%%% Purpose : 
%%% Created : 28 Jan 1997 by Christofer Toernkvist <chris@jones>
%%%----------------------------------------------------------------------

-module(mod_listbox).
-copyright('Copyright (c) 1991-97 Ericsson Telecom AB').
-vsn('$Revision: /main/release/london/2 $ ').    %'
-author('chris@jones').

%%-compile(export_all).
-export([new/0,
	 toggle_tag/1,
	 get_modules/1,
	 get_indices/1,
	 push_modules/2,
	 push_indices/2,
	 set_rbuttons_fg/1,
	 update/4,
	 delete_loaded_modules/1,
	 toggle_display/1]).

%% EXTERNAL FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%

new() -> {showdef, { {showdef, [], []}, {showundef,[], []} }  }.

toggle_tag({showdef,   Data}) -> {showundef, Data};
toggle_tag({showundef, Data}) -> {showdef, Data}.
    
get_modules({showdef,   {{showdef, Modules, _}, _    }  }) -> Modules;
get_modules({showundef, { _, {showundef, Modules, _} }  }) -> Modules.

get_indices({showdef, {{showdef, _, Indices}, _     } }) -> Indices;
get_indices({showundef, { _, {showundef, _, Indices}} }) -> Indices.

push_modules({showdef, { {showdef, _, Indices}, Data}}, Modules) ->
    {showdef, { { showdef, Modules, Indices}, Data}};
push_modules({showundef, { Data, {showundef, _, Indices}}}, Modules) ->
    {showundef, { Data, { showundef, Modules, Indices}}}.

push_indices({showdef, { {showdef, Modules, _}, Data}}, Indices) ->
    {showdef, { { showdef, Modules, Indices}, Data}};
push_indices({showundef, { Data, {showundef, Modules, _}}}, Indices) ->
    {showundef, { Data, { showundef, Modules, Indices}}}.

toggle_display({showdef, { {showdef, Def_mod, Inx_def}, {showundef, Undef_mod, Inx_undef}}}) ->
    {Disp_mod, Inx_disp, Hide_mod, Inx_hide} = display(Undef_mod, Inx_undef, Def_mod, Inx_def),
    {showundef, {{showdef, Hide_mod, Inx_hide}, {showundef, Disp_mod, Inx_disp}}};
toggle_display({showundef, { {showdef, Def_mod, Inx_def}, {showundef, Undef_mod, Inx_undef}}}) ->
    {Disp_mod, Inx_disp, Hide_mod, Inx_hide} = display(Def_mod, Inx_def, Undef_mod, Inx_undef),
    {showdef, {{showdef, Disp_mod, Inx_disp}, {showundef, Hide_mod, Inx_hide}}};
toggle_display({ A , { {showdef, B, C } , {showundef, D , E }}}) -> true;
toggle_display({ A , { B , C }}) -> true;
toggle_display( Data ) -> true.


set_rbuttons_fg({showdef, {{showdef, Def_mod, _}, {showundef, Undef_mod, _}}}) ->
    srb_fg(Def_mod, Undef_mod);
set_rbuttons_fg({showundef, {{showdef, Def_mod, _}, {showundef, Undef_mod, _}}}) ->
    srb_fg(Def_mod, Undef_mod).
srb_fg(Def_mod, Undef_mod) ->
    rb_mod:set_greyscale_fg(showdef,   Def_mod),
    rb_mod:set_greyscale_fg(showundef, Undef_mod).


update(Def_mod,       %% [atom1, atom2, ...]
       Undef_mod,     %% [atom1, atom2, ...]
       New_selected,  %% [atom1, atom2, ...]
       {showdef, {{showdef, Mod_def, Inx_def}, {showundef, Mod_undef, Inx_undef}}}) ->
    %%
    %% DEF - dealing with the defined module part.
    %%
    Current_selected = lists:foldl(fun ( N , Acc ) ->    %% <-- (Inx_def --> Mod_def)
				       [lists:nth(N + 1, Mod_def) | Acc] end, [], Inx_def),
    All_selected     = lists:append(New_selected, Current_selected),
    ok = gs:config(modlb, {items, Def_mod}),
    highlight(All_selected),
    New_inx_def = gs:read(modlb, selection),
    %%
    %% UNDEF - dealing with the undefined module part.
    %%
    Curr_sel_undef = lists:foldl(fun ( N , Acc ) ->    %% <-- (Inx_undef --> Mod_undef)
				       [[lists:nth(N + 1, Mod_undef)] | Acc] end, [], Inx_undef),

    New_inx_undef  = get_indices_of(Curr_sel_undef, Undef_mod),
    {showdef, {{showdef, Def_mod, New_inx_def}, {showundef, Undef_mod, New_inx_undef}}};
update(Def_mod,
       Undef_mod,
       New_selected,
       {showundef, {{showdef, Mod_def, Inx_def}, {showundef, Mod_undef, Inx_undef}}}) ->
    %%
    %% UNDEF - dealing with the undefined module part.
    %%
    Current_selected = get_names_on_indices(gs:read(modlb, selection)),
    New_inx_undef    = get_indices_of(Current_selected, Undef_mod),
    ok = gs:config(modlb, {items, Undef_mod}),  %% display undef modules.
    lists:foreach(fun ( Index ) -> ok = gs:config(modlb, {selection, Index}) end, New_inx_undef), %% highlight
    %%
    %% DEF - dealing with the defined module part.
    %%
    Curr_sel_def = lists:flatten(lists:foldl(fun ( N , Acc ) ->    %% <-- (Inx_def --> Mod_def)
				       [[lists:nth(N + 1, Mod_def)] | Acc] end, [], Inx_def)),
    Selected_def = lists:append(New_selected, Curr_sel_def),
    New_inx_def  = get_indices_of(Selected_def, Def_mod),
    {showundef, {{showdef, Def_mod, New_inx_def}, {showundef, Undef_mod, New_inx_undef}}};
update( A1 ,
        A2 ,
        A3 ,
        A4 ) -> {showdef, {{showdef, [], []}, {showundef, [],[]}}}.


delete_loaded_modules({showdef, Data}) ->
    %% 1. Get selected.
    %% 2. Get loaded of the selected.
    %% 3. Exclude out of EXREF.
    %% 4. Analyse with EXREF.
    %% 5. Display.
    Current_selected = gs:read(modlb, selection),
    [Left_curly_bracket] = atom_to_list('{'),
    Loaded_selected  = lists:foldl(fun ( Index , Acc) ->
					   case gs:read(modlb, {get, Index}) of
					       [Left_curly_bracket|T] ->
						   Acc;
					       Name ->
						   exref:delete_module(list_to_atom(Name)),
						   [Name | Acc]
					   end
				   end,
				   [],
				   Current_selected),
    case Loaded_selected of
	[] ->
	    {showdef, Data};     %% return value
	Other ->
	    {undefined_functions, Modules, Undefined_modules} = exref:analyse(undefined_functions),
	    {Def_mod, Lib_mod, Undef_mod} = xref_gp:divide_in_mod_types(Modules, Undefined_modules),
	    %%CT%%New_modlb_show = mod_listbox:update(lists:append(Def_mod, Lib_mod),
	    Mod_def = lists:append(Def_mod, Lib_mod),
	    ok = gs:config(modlb, {items, Mod_def}),
	    rb_mod:set_greyscale_fg(showdef,   Mod_def), 
	    rb_mod:set_greyscale_fg(showundef, Undef_mod), 
	    xref:set_funcframe([]),
	    {showdef, {{showdef, Mod_def, []}, {showundef, Undef_mod, []}}}  %% return value
    end;
delete_loaded_modules({showundef, Data}) -> {showundef, Data}.

	    
    			  
			     
%% INTERNAL FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%% 

display(Disp_mod, Inx_disp, Hide_mod, Inx_hide) ->
    New_inx_hide = gs:read(modlb, selection),
    ok = gs:config(modlb, clear),
    ok = gs:config(modlb, {items, Disp_mod}),
    lists:foreach(fun ( Index ) -> ok = gs:config(modlb, {selection, Index}) end, Inx_disp),
    {Disp_mod, Inx_disp, Hide_mod, New_inx_hide}.


%% highlight/1
%%
%% Purpose:  highlights the newly selected modules and the modules which already were
%%           highlighted, in modul-listbox.
%%
%% Arg:      [atom1, atom2,...]
%%
highlight(Mod_to_select) ->
    Do_select_in_modlb = fun ([]) ->  %% there where no index, i.e. size=0
				  true;
			      (Index) ->    %% Foreach shown module, check if it should be highlighted.
				  [Name_of_index] = get_names_on_indices([Index]), %% <-- [atom]
				  case lists:member(Name_of_index, Mod_to_select) of
				      false ->
					  true;
				      true ->
					  ok = gs:config(modlb, {selection, Index}) %% used side-effect.
				  end
			  end,
    lists:foreach(Do_select_in_modlb, get_modlb_indices()). %% foreach item in mod-listbox

get_modlb_indices() ->
    case gs:read(modlb, size) of
	0 ->         [];
	Number_of -> lists:seq(0, Number_of - 1)
    end.

get_indices_of(Elist, Name_list) ->	
    gi_of_l(Elist, Name_list, []).

gi_of_l([], _, Acc) -> Acc;
gi_of_l(_, [], Acc) -> Acc;
gi_of_l([H|T], Name_list, Acc) ->
    case gi_of_i(H, Name_list, 0) of
	not_found ->
	    gi_of_l(T, Name_list, Acc);
	Index ->
	    gi_of_l(T, Name_list, [Index | Acc])
    end.

gi_of_i(Name, [Name|T],  Inx) -> Inx;
gi_of_i(_, [],  Inx) -> not_found;
gi_of_i(Name, [_|T], Inx) ->
    gi_of_i(Name, T, Inx + 1).


%% get_names_on_indices/1
%%
%% Args: List of indices of items in a graphical listbox.
%%
%% Ret:  List of names (as strings) corresponding to the listbox indices.
%%
get_names_on_indices([]) ->
    [];
get_names_on_indices(Indices) ->
    Get_name_of_index = fun ( Index ) ->
				Text = gs:read(modlb, {get, Index}), %% -> "name"
				case parse(Text) of
				    {_, Mod } ->
					Mod;
				    Mod_name ->
					Mod_name
				end
			end, 
    lists:map(Get_name_of_index, Indices). %% -> names of all selected modules.

parse(Txt) ->
    String = lists:append(Txt,". "),
    {ok,Ts,_} = erl_scan:string(String),
    {ok,Name} = erl_parse:parse_term(Ts),
    Name.


