%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%%----------------------------------------------------------------------
%%% File    : rb_mod.erl
%%% Author  : Christofer Toernkvist <chris@jones>
%%% Purpose : Interface the radiobuttons data structure.
%%% Created : 12 Jan 1997 by Christofer Toernkvist <chris@jones>
%%%----------------------------------------------------------------------

-module(rb_mod).
-copyright('Copyright (c) 1991-97 Ericsson Telecom AB').
-vsn('$Revision: /main/release/london/1 $ ').   %'
-author('chris@jones').

-export([show_modules/3,
	 init/0,
	 push/2,
	 set_greyscale_fg/2,
	 set_greyscale_fg/1,
	 toggle_select/2,
	 is_selected/1]).

-define(Modules, module_tag).
-define(Selection_indices, sel_indices_tag).
-define(Gs_grey, 163). 
-define(Gs_black,  0). 


%% INTERFACE FUNCTIONS

init() ->
    gs:config(showdef,   {select, true}),
    set_greyscale_fg(showdef, grey),
    gs:config(showdef,   {data, selected}),
    set_greyscale_fg(showundef, grey),
    gs:config(showundef, {data, non_selected}).

toggle_select(Rbutton, Data) ->
    case is_selected(Data) of 
	true ->
	    ok = gs:config(Rbutton, {data, non_selected}),
	    ok = gs:config(opposite_button(Rbutton), {data, selected});	
	false ->
	    ok = gs:config(Rbutton, {data, selected}),
	    ok = gs:config(opposite_button(Rbutton), {data, non_selected})
	end.
	    

is_selected(selected)     -> true;
is_selected(non_selected) -> false.

set_greyscale_fg(Button, grey) ->
    gs:config(Button, {fg, {?Gs_grey, ?Gs_grey, ?Gs_grey}});
set_greyscale_fg(Button, black) ->
    gs:config(Button, {fg, {?Gs_black, ?Gs_black, ?Gs_black}});
set_greyscale_fg(Button, []) ->
    gs:config(Button, {fg, {?Gs_grey, ?Gs_grey, ?Gs_grey}});
set_greyscale_fg(Button, L) when list(L) ->
    gs:config(Button, {fg, {?Gs_black, ?Gs_black, ?Gs_black}}).

set_greyscale_fg(Modlb_show) ->
    mod_listbox:set_rbuttons_fg(Modlb_show).


show_modules(Button, {non_selected, {?Modules, Modules}, {?Selection_indices, Hl}}, Skip_this) ->
    save_status_module_listbox(opposite_button(Button), non_selected),
    gs:config(Button, {data, data_structure(selected, [],[])}),
    %%CT%%gs:config(Button, {value, selected}),
    %%CT%%gs:config(opposite_button(Button), {value, non_selected}),
    gs:config(modlb, {items, Modules}),
    lists:foreach(fun highlight_index/2, [modlb], Hl);
show_modules(_, _, _) ->
    true.

push(Undef_mod, showundef) ->
    gs:config(showundef, {data, data_structure(non_selected, Undef_mod, [])});
push(Def_mod, showdef) ->
    gs:config(showdef, {data, data_structure(non_selected, Def_mod, [])}).
    

%% INTERNAL FUNCTIONS

data_structure(Choice, Modules, Highlight_indices) ->
    {Choice, {?Modules, Modules},{?Selection_indices, Highlight_indices}}.

opposite_button(showdef)   -> showundef;
opposite_button(showundef) -> showdef.

highlight_index(Index, Listbox) ->
    gs:config(Listbox, {selection, Index}).

save_status_module_listbox(Button_name, Choice) ->
    Modules           = get_module_names(modlb, gs:read(modlb, size)),
    Highlight_indices = gs:read(modlb, selection),
    gs:config(Button_name, {data, data_structure(Choice, Modules, Highlight_indices)}).

get_module_names(Listbox, 0) ->
    [];
get_module_names(Listbox, Nr_of_indices) ->
    lists:map(fun get_name_of_index/2, [Listbox], lists:seq(0, Nr_of_indices - 1)).

get_name_of_index(Index, Listbox) ->
    gs:read(Listbox, {get, Index}).


