%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.0, (the "License"); you may not use this file except in
%% compliance with the License. You may obtain a copy of the License at
%% http://www.erlang.org/EPL1_0.txt
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Original Code is Erlang-4.7.3, December, 1998.
%% 
%% The Initial Developer of the Original Code is Ericsson Telecom
%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%% Telecom AB. All Rights Reserved.
%% 
%% Contributor(s): ______________________________________.''
%%
%%% -------------------------------------------------------------------
%%% File    : xref.erl
%%% Author  : Maria Holm 
%%% Purpose : To draw a the graph returned from Exref on a gs:canvas
%%% Created : August 1996 by Maria Holm
%%% Edited  : Februari 1997 by Christofer Tornkvist
%%% $Id: xref.erl,v 1.2 1996/12/11 10:51:29 chris Exp chris $
%%% -------------------------------------------------------------------
%%% Edit comments:
%%%
%%% -QUIT   When QUIT is invoked all spawned processes (windows)
%%%         should be stopped. This is for the moment done by the
%%%         precess_flag(trap_exit, true) mechanism, which is
%%%         very uggly. This should insted be acomplished by sending an EXIT signal.
%%%         
%%% -Bugs   In EXREF.ERL there are 2 bugs:
%%%         1. All modules belonging to a certain application is loaded,
%%%            which implies no undefined modules (actually func.).
%%%            One or several modules is deleted, but still there are no undefined
%%%            modules ?!
%%%         2. One module is loaded, which imply some undefined modules.
%%%            Delete the module, but still the undefined moudles remains ?!
%%%         In XREF.ERL there is at least 2 bugs:
%%%         1. When level > 0 some functions will not by displayed in the
%%%            graph-window at start, but will appear if a reference belonging to
%%%            that module is to be shown. A correct graph will on the other hand
%%%            be shown if level == 0 and functions is forced to appear by the extend-button.
%%%         2. In the functions section, above the rightmost listbox is a label. The label
%%%            shows which module all the functions in the listbox belong to. Within the 
%%%            program this label is refered to as an entry (funcentry) which is very missleading.
%%%            The label should be called something like funclabel. The change should be
%%%            made in the resource file xref_main.rc edited in the GUI-builder.
%%%
%%% -Selecting In the listboxes mouse-drag-selecting should be possible.
%%%
-module(xref).
-copyright('Copyright (c) 1991-97 Ericsson Telecom AB').
-vsn('$Revision: /main/release/free/1').
-export([start/0,init/0,main_loop/2,show_graph/4, set_funcframe/1]).

-define(Ov,5).
-define(Left_button,   1).   %% on mouse
-define(Default_level, 2).   %% default value of level (levelentry)
-define(GraphWinHead, "Xref - graph : ").

-include("vertex.hrl").      %% containded in every graph-node
-include("edge.hrl").        

start() ->
    spawn(xref,init,[]).

init() ->
    exref:start(),
    {ok,Dir} = file:get_cwd(),      % gets the Dir to send to the file dialog
    xref_main_ui:start(gs:start()),
    file_dial:refresh(Dir),
    ok = gs:config(levelentry,{text, ?Default_level}),   %% sets the defaul value of levels
    ok = gs:config(win,{map,true}),
    rb_mod:init(),                         %% radiobuttons default status
    main_loop(Dir, mod_listbox:new()).

%% main_loop/2
%%
%% Purpose: taking care of some basic events like
%%          - moving to one of the frames Files, Modules and Functions
%%          - invoking the functions Locals-not-called, Exports-not-called, Recursive-modules
%%
%% Arg:     Dir        = "directory_path"
%%          Modlb_show = data saying which of the two categories
%%                       selected-modules plus LIB-modules and undefined-modules that
%%                       should be displayed in the modules listbox (modlb).
%%                       { name-of-radiobutton-currently-selected,
%%                         {name-rb-1, [module-name-1, ...], [selected_index_1,...]},
%%                         {name-rb-2, [undefined-mod-name-1,...], [selected_index_1,...]}
%%                       }
%%
main_loop(Dir, Modlb_show) ->
    receive
	{gs,fileframe,enter, _, _} ->
	    file_dial:fileloop(Dir, Modlb_show);
	{gs,modframe,enter, _, _} ->
	    New_modlb_show = modloop(Dir, Modlb_show),
	    main_loop(Dir, New_modlb_show);
	{gs,funcframe,enter, _, _} ->
	    funcloop(),
	    main_loop(Dir, Modlb_show);
	{gs,loc,click,_,_} ->
	    Op_type = locals_not_called,
	    Exref = exref:analyse(Op_type),
	    spawn_link(xref_gp,ed_disp,[Exref,Op_type,self()]),
	    main_loop(Dir, Modlb_show);
	{gs,exp,click,_,_} ->
	    Op_type = exports_not_called,
	    Exref = exref:analyse(Op_type),
	    spawn_link(xref_gp,ed_disp,[Exref,Op_type,self()]),
	    main_loop(Dir, Modlb_show);
	{gs,rec,click,_,_} ->
	    Op_type = recursive_modules,
	    Exref = exref:analyse(Op_type),
	    spawn_link(xref_gp,ed_disp,[Exref,Op_type,self()]),
	    main_loop(Dir, Modlb_show);
	{gs,quit,click,_,_} ->
	    exref:stop(),
	    gs:destroy(win),
	    exit(normal);
	Other ->
	    main_loop(Dir, Modlb_show)
    end.

%% modloop/2
%%
%% Purpose: taking care of events in the module-frame.
%%
%% Returns: -
%%
%% Args   : (see main_loop/2)
%%
modloop(Dir, Modlb_show) ->
    receive
	{gs, Radiobutton, click, Data, [_, showmod, Value|_]} ->
	    New_modlb_show = case rb_mod:is_selected(Data) of
				 true ->				     
				     Modlb_show;
				 false ->
				     change_mod_categori(Radiobutton, Data, Modlb_show)
			     end,
	    modloop(Dir, New_modlb_show);
	{gs, modlb, click, _, [Idx,Txt, Is_selected | _]} ->
	    update_funcframe(Idx, Txt, Is_selected, gs:read(modlb, selection)),
	    New_modlb_show = mod_listbox:push_indices(Modlb_show, gs:read(modlb, selection)),
	    modloop(Dir, New_modlb_show);
	{gs, delmod, click, _, _} ->
	    New_modlb_show = mod_listbox:delete_loaded_modules(Modlb_show),
	    modloop(Dir, New_modlb_show);
	{gs, delall, click, _, _} ->
	    exref:stop(),
	    exref:start(),
	    ok = gs:config(funclb,{items,[]}),
	    ok = gs:config(funcentry,{label, {text, ""}}),    
	    ok = gs:config(modlb, clear),
	    rb_mod:init(),

% Adding modules when undefined modules are shown in the "module
% listbox" causes crashes.
% This is remedied by disallowing the described operation. When
% the "undefined" button is clicked, the "load" and "delete module"
% buttons are disabled. Choosing ""defined" enables these buttons
% again.Pressing the "delete all" button has the side effect of
% setting the states of the radiobuttons so that "defined" is selected.
% In accordance with this, the "load" and "delete module" buttons
% must be enabled then. The underlying bug is not fixed.
	    gs:config(load, {enable, true}),
	    gs:config(delmod, {enable, true}),

	    modloop(Dir, mod_listbox:new());

	{gs,modview,click,_,_} ->
	    Indx = gs:read(modlb,selection),
	    Mods = lists:map(fun(Idx) ->
				    Txt = gs:read(modlb,{get,Idx}),
				    case parse(Txt) of
					{_,Mod} -> Mod;
					Other   -> Other
				    end
			    end,Indx),
	    Level = case gs:read(levelentry,text) of 
			[] -> infinity;
			Other -> list_to_integer(Other)
		    end,
	    case Mods of
		[] ->
		    true;
		[H|[]] ->  
		    Win_title = lists:append([?GraphWinHead,
					     " study of ",
					     integer_to_list(1),
					     " certain module."]),
		    spawn_link(xref,show_graph,[Mods,Level,self(), Win_title]);
		_List ->  
		    Win_title = lists:append([?GraphWinHead,
					     " study of ",
					     integer_to_list(length(Mods)),
					     " certain modules."]),
		    spawn_link(xref,show_graph,[Mods,Level,self(), Win_title])
	    end,
	    modloop(Dir, Modlb_show);
	{gs,modframe,leave,_,_} ->
	    Modlb_show;
	Other ->
	    modloop(Dir, Modlb_show)
    end.

%% change_mod_categori/3
%%
%% Purpose: Change the modules to be displayed in the module-listbox.
%%          That is, display either defined modules or undefinned modules.
%%
%% Returns: {What_to_show_TAG,
%%            { {defined_TAG,   [,defined_modules],   [,selected_indices]}
%%              {undefined_TAG, [,undefined_modules], [,selected_indices]}}}
%%
change_mod_categori(Radiobutton, Data, Modlb_show) ->
% Adding modules when undefined modules are shown in the "module
% listbox" causes crashes.
% This is remedied by disallowing the described operation. When
% the "undefined" button is clicked, the "load" and "delete module"
% buttons are disabled. Choosing "defined" enables these buttons
% again.Pressing the "delete all" button has the side effect of
% setting the states of the radiobuttons so that "defined" is selected.
% In accordance with this, the "load" and "delete module" buttons
% must be enabled then. The underlying bug is not fixed.
    if
	Radiobutton == showundef ->
	    gs:config(load, {enable, false}),
	    gs:config(delmod, {enable, false});
	Radiobutton == showdef ->
	    gs:config(load, {enable, true}),
	    gs:config(delmod, {enable, true})
    end,

    Size = gs:read(funclb, size),
    rb_mod:toggle_select(Radiobutton, Data),
    New_modlb_show = mod_listbox:toggle_display(Modlb_show),
    case Size of
	0 ->
	    Selections = gs:read(modlb, selection),
	    case Selections of
		[]            -> true;
		[Top_index|_] ->
		    update_funcframe(Top_index, gs:read(modlb, {get, Top_index}), true, Selections)
	    end;
	Other ->
	    true
    end,
    New_modlb_show.

%% update_funcframe/4   (Block of functions)
%%
%% Type of: Procedure, i.e. only using side effects.
%%
%% Purpose: If a new module has been selected in the module list (modlb)
%%          show the corresponding function info. in the funclb listbox.
%%          If a selected module has been diselected in the module list
%%          show the function info. from the top module in the list.
%%          If no module is selected empty funcentry field and funclb.
%%
%% Arg:     Index := integer <==> ordernumber in module list.
%%          Text  := selected/unselected module name.
%%          bool  := selected -> true ; unselected -> false.
%%          list  := selection index list.
%%
%% Help func:   set_funcframe/1
%%
update_funcframe(Index, _, true, _) ->      %% Module with index Index has been selected.
    Modname = gs:read(modlb, {get, Index}),
    set_funcframe(Modname);
update_funcframe(_, _, false, []) ->        %% No module is selected.
    set_funcframe([]);
update_funcframe( _, Text, false, [Top_index|_]) ->  %% the sel. mod. shown in funcframe label was unsel.
    case gs:read(funcentry, {label, text}) of
	{text, Text} ->  %% remove the unselected module name.
	    Modname = gs:read(modlb, {get, Top_index}),
	    set_funcframe(Modname);
	{text, Not_text} ->  
	    true; 
	Fe ->    
	    case Text == lists:flatten(Fe) of  %% due to the answer (CT?!) above (text of a label) 
		true ->
		    Modname = gs:read(modlb, {get, Top_index}),
		    set_funcframe(Modname);
		false ->
		    true
	    end
    end.

%% set_funcframe/1
%%
%% Purpose: set the funcentry field and the funclb list due
%%          to some chnages of the selected modules in module-listbox.
%%
set_funcframe([]) ->
    ok = gs:config(funclb,{items,[]}),
    ok = gs:config(funcentry, {label, {text, ""}});
set_funcframe(Modname) ->
    ok = gs:config(funcentry, {label, {text, Modname}}),
    xref_gp:disp_funcs(Modname).
%%
%% End update_funcframe/2    

%% funcloop/0
%%
%% Purpose: handle events in the functions-frame, that is
%%          events for the functions-listbox, button view-functions,
%%          and the speciell event "enter" for the level-entry.
%%
%% Returns: nothing
%%
%% Args   : none
%%
funcloop() ->
    receive
	{gs,funcview,click,_,_} ->
	    Level = case gs:read(levelentry,text) of
			[]    -> infinity;
			Other -> list_to_integer(Other)
		    end,
	    Indx = gs:read(funclb,selection),
	    Funcs = lists:map(fun(Idx) ->
				     Func = gs:read(funclb,{get,Idx}),
				     parse(Func)
			    end,Indx),
	    {text, Mod} = gs:read(funcentry, {label, text}),
	    case Funcs of 
		[]      ->
		    funcloop();
		[H|[]]  ->
		    Win_title = lists:append([?GraphWinHead,
					     " study of ",
					     integer_to_list(1),
					     " function in module ",
					     Mod]),
		    spawn_link(xref,show_graph,[Funcs,Level,self(), Win_title]);
		_List  ->
		    Win_title = lists:append([?GraphWinHead,
					     " study of ",
					     integer_to_list(length(Funcs)),
					     " functions in module ",
					     Mod]),
		    spawn_link(xref,show_graph,[Funcs,Level,self(), Win_title])
	    end;
	{gs, funcframe, leave,_,_} ->
	    ok;
	{gs, levelentry, enter, _, _} ->
	    levelloop(gs:read(levelentry, text)),
	    funcloop();
	Other ->
	    funcloop()
    end.

%% levelentry/1
%%
%% Purpose: handle the user interaction setting the level in the level-entry field.
%%
%% Arg:     holds the level value that was valid before entering the level-entry field.
%%
levelloop(Current_value) ->
    receive
	{gs, levelentry, click, _, _} ->
	    gs:config(levelentry, {setfocus, true});
	{gs, levelentry, leave, _, _} ->
	    Levels = gs:read(levelentry, text),
	    case catch list_to_integer(Levels) of
		{'EXIT', _} -> ok = gs:config(levelentry, {text, Current_value});
		Int         -> true
	    end,
	    ok = gs:config(levelentry, {setfocus, false});
	Other ->
	    levelloop(Current_value)
    end.

%% parse/1
%%
%% Purpose: transform text.
%%
%% Returns: parse("test_text") --> test_text
%%             where atom(test_text) --> true
%%
parse(Txt) ->
    String = lists:append(Txt,". "),
    {ok,Ts,_} = erl_scan:string(String),
    {ok,Name} = erl_parse:parse_term(Ts),
    Name.



%% show_graph/3
%%
%% Purpose: takes care of interaction for the graphwindow
%%
%% Arg:  Name  := a list with [function | module]  names. 
%%       Level := how deep connection analysis that will be shown.
%%       Owner := pid.
%%       Win_title := Window title
%%
show_graph(Names,Level,Owner, Win_title) ->
    process_flag(trap_exit, true),
    put(n,0),   % otp-p1c bug, this is needed for graph_placer
    {A,B,C} = now(),
    random:seed(A,B,C),
    xref_graph_ui:start(gs:start()),
    gs:create(canvas,canvas,graphwin,[{width,663},{height,420},
				      {x,5},{y,5},{vscroll,right},
				      {hscroll,bottom},{motion,true},
				      {scrollregion,{-900,-800,900,800}}]),
    ok = gs:config(graphwin, {title, Win_title}),
    ok = gs:config(canvas,[{leave,true},{buttonpress,true}]),

    Graph = digraph:new(),
    ok = gs:config(graphwin,{cursor,busy}),
    xref_gp:create_base(Graph,Names,Level),
    ok = gs:config(graphwin,{cursor,arrow}),
    graph_loop(Graph,no_selection,Owner).

%% graph_loop/3
%%
%% Purpose: handle events in the graph-window.
%% 
%% Arg:  Name  := a list with [function | module]  names. 
%%       Level := how deep connection analysis that will be shown.
%%       Owner := pid.
%%
graph_loop(G,Sel,Owner) ->
    receive
	{'EXIT', Owner, Reason} ->
	    exit(normal);  %% together with trap_exit detect if QUIT is pushed in main window.
	{gs, graphwin ,leave,_,_} ->
	    Owner ! {graphwin,leave},
	    graph_loop(G,Sel,Owner);
	{gs, close ,click,_,_} ->
	    digraph:delete(G),
	    gs:destroy(graphwin);
	{gs,  canvas  ,buttonpress,_,_} ->    %unselects node
	    case Sel of
		{Node,Vertex,true,Name} ->
		    ok = gs:config(Node,{fg,yellow}),
		    ok = gs:config(nodeentry,{text,""}),
		    ok = gs:config(fix,{select,false});
		{Node,Vertex,false,Name} ->
		    ok = gs:config(Node,{fg,red}),
		    ok = gs:config(nodeentry,{text,""}),
		    ok = gs:config(fix,{select,false});
		_ -> ok
	    end,
	    graph_loop(G,no_selection,Owner);
	{gs, relax ,click,_,_} ->
	    ok = gs:config(relax,{select,true}),
	    relax(G,false),
	    graph_loop(G,Sel,Owner);
	{gs, graphextend ,click,_,_} -> 
	    ok = gs:config(graphwin,{cursor,busy}),
	    xref_gp:extend(G,digraph:vertices(G),all),
	    ok = gs:config(graphwin,{cursor,arrow}),
	    graph_loop(G,Sel,Owner);
	{gs, showall ,click,_,_} ->
	    ok = gs:config(graphwin,{cursor,busy}),
	    xref_gp:show_edges(G),
	    ok = gs:config(graphwin,{cursor,arrow}),
	    graph_loop(G,Sel,Owner);
	{gs, hideall ,click,_,_} ->
	    ok = gs:config(graphwin,{cursor,busy}),
	    xref_gp:destroy_edges(G),
	    ok = gs:config(graphwin,{cursor,arrow}),
	    graph_loop(G,Sel,Owner);
%	{gs, nodeentry , enter, _, _} ->
%	    Txt = gs:read(nodeentry,text),
%	    NewS = xref_gp:select_node(G, Sel, Txt),
%	    gs:config(nodeentry, {setfocus, true}),
%
%	    NewS = nodeentry_loop(G, Sel),
%	    graph_loop(G,NewS,Owner);
	{gs, Node , enter,{digraph_vertex,Gid},_} ->  %% Node = {Number,Pid}
	    case Sel of
		no_selection ->
		    {_,D} = digraph:vertex(G,Gid),
		    #vertex{x=X,y=Y,fixed=Fixed,data=[Name]}=D,
		    ok = gs:config(nodeentry,{text, Name}),
		    ok = gs:config(fix,{select,Fixed});
		Other ->
		    ok
	    end,
	    graph_loop(G,Sel,Owner);
	{gs, Node , buttonpress,{digraph_vertex,Gid},[?Left_button,X,Y|Rest]} ->
	    NewSel = case Sel of
			 {GsId,Vertex,true,Name} ->
			     ok = gs:config(GsId,{fg,yellow}),
			     select(G,Node,X,Y,Gid);
			 {GsId,Vertex,false,Name} ->
			     ok = gs:config(GsId,{fg,red}),
			     select(G,Node,X,Y,Gid);
			 Other ->
			     select(G,Node,X,Y,Gid)
		     end,
	    graph_loop(G,NewSel,Owner);
	{gs, fix ,click,_,_} ->
	    case Sel of
		{Node,Gid,Fixed,Name} ->
		    {_,V} = digraph:vertex(G,Gid),
		    V2 = V#vertex{fixed=inv(Fixed)},
		    digraph:add_vertex(G,Gid,V2),
		    ok = gs:config(fix,{select,inv(Fixed)}),
		    graph_loop(G,{Node,Gid,inv(Fixed),Name},Owner);
		_ -> ok = gs:config(nodeentry,{text,"Must select node"}),
		     graph_loop(G,Sel,Owner)
	    end;
	{gs,showout,click,_,_} ->
	    ok = gs:config(graphwin,{cursor,busy}),
	    case Sel of
		no_selection ->
		    ok = gs:config(nodeentry,{text,"Must select node"});
		{Node,Gid,Fixed,Name} ->
		    xref_gp:extend(G,[Gid],out)
	    end,
	    ok = gs:config(graphwin,{cursor,arrow}),
	    graph_loop(G,Sel,Owner);
	{gs,hideout,click,_,_} ->
	    ok = gs:config(graphwin,{cursor,busy}),
	    case Sel of 
		no_selection ->
		    ok = gs:config(nodeentry,{text,"Must select node"});
		{Node,Gid,Fixed,Name} ->
		    lists:foreach(fun (Eid) ->
					 {_,_,_,D} = digraph:edge(G,Eid),
					 #edge{gsid = Edge} = D,
					 gs:destroy(Edge),
					 digraph:del_edge(G,Eid)
				 end,digraph:out_edges(G,Gid))
	    end,
	    ok = gs:config(graphwin,{cursor,arrow}),
	    graph_loop(G,Sel,Owner);
	{gs,showin,click,_,_} ->
	    ok = gs:config(graphwin,{cursor,busy}),
	    case Sel of
		no_selection ->
		    ok = gs:config(nodeentry,{text,"Must select node"});
		{Node,Gid,Fixed,Name} ->
		    xref_gp:extend(G,[Gid],in)
	    end,
	    ok = gs:config(graphwin,{cursor,arrow}),
	    graph_loop(G,Sel,Owner);
	{gs,hidein,click,_,_} ->
	    ok = gs:config(graphwin,{cursor,busy}),
	    case Sel of 
		 no_selection ->
		    ok = gs:config(nodeentry,{text,"Must select node"});
		{Node,Gid,Fixed,Name} ->
		    lists:foreach(fun (Eid) ->
					 {_,_,_,D} = digraph:edge(G,Eid),
					 #edge{gsid = Edge} = D,
					 gs:destroy(Edge),
					 digraph:del_edge(G,Eid)
				 end,digraph:in_edges(G,Gid))
	    end,
	    ok = gs:config(graphwin,{cursor,arrow}),
	    graph_loop(G,Sel,Owner);
	Other ->
	    graph_loop(G,Sel,Owner)
    end.

nodeentry_loop(G, Sel) ->
    receive
	{'EXIT', Owner, Reason} ->
	    %% together with trap_exit, this detects if QUIT is pushed in main window.
	    exit(normal); 
 	{gs, nodeentry, keypress, _Data, ['Return'|_]} ->
	    Text  = gs:read(nodeentry, text),
	    NewSel = xref_gp:select_node(G, Sel, Text),
	    nodeentry_loop(G, NewSel);
	{gs, nodeentry, leave, _, _} ->
	    ok = gs:config(nodeentry, {setfocus, false}),
	    Sel;
	Other ->
	    nodeentry_loop(G, Sel)
    end.

relax(G,Relax) ->
    receive
	{gs,relax,click,_,_} ->
	    ok = gs:config(relax,{select,false});
	Other ->
	    relax(G,Relax)
    after 1000 ->
	    case Relax of
		false ->
		    {NewR,_} = graph_placer:relax(G,2,0.5,{600,600}),
		    graph_placer:draw(G),
		    relax(G,NewR);
		true ->
		    ok = gs:config(relax,{select,false})
	    end
    end.


inv(true) -> false;
inv(false) -> true.


%% drag/3
%%
%% Purpose: makes it possable to drag a node by holding
%%          the mousebutton down.
%% 
%% Arg:  G    = graph
%%       Node = graphical nodeobject
%%       {Mouse_x, Mouse_y} = mouse coords. relative to the 
%%                            anchor point of the nodeobject.
%%
drag(G,Node, {Mouse_x, Mouse_y}) ->
    receive
	{gs,canvas,motion,_,[X,Y|_]} ->
	    ok = gs:config(Node,{coords,[{X-Mouse_x,Y-Mouse_y}]}),
	    drag(G,Node, {Mouse_x, Mouse_y});
	{gs,Node,buttonrelease,_,[_,X,Y|_]} ->
	    ok = gs:config(Node,{coords,[{X-Mouse_x,Y-Mouse_y}]}),
	    {X-Mouse_x,Y-Mouse_y};
	Other ->
	    drag(G,Node, {Mouse_x, Mouse_y})
    end.


%% select/5
%%
%% Purpose: when the mousebutton is pushed the node
%%          is selected (green color). The mousedrag facility
%%          is also initiated.
%% 
select(G,Node,X,Y,Gid) ->   %selects and moves node
    ok = gs:config(Node,{fg, green}),
    {_,V} = digraph:vertex(G,Gid),
    #vertex{data=[Node_name],fixed=Fixed} = V,

    case Node_name of
	{Mod, Func, Arity} ->
	    Tmp1 = lists:append([atom_to_list(Mod),[$:],atom_to_list(Func),[$/],integer_to_list(Arity)]),
	    Name = list_to_atom(Tmp1);
	Name ->
	    Name
    end,

    ok = gs:config(nodeentry,{text,Name}),
    ok = gs:config(fix,{select,Fixed}),
    Mouse_x  = X - V#vertex.x, %% difference to mouse position
    Mouse_y  = Y - V#vertex.y,
    {Mx,My} = drag(G, Node, {Mouse_x, Mouse_y}),
    V2 = V#vertex{x = Mx,y = My},
    Test_vertex = digraph:add_vertex(G,Gid,V2),
    graph_placer:draw(G),
    {Node,Gid,Fixed,Name}.




